@@ -17,9 +17,11 @@
 
 package org.apache.commons.configuration;
 
+import java.lang.reflect.Array;
 import java.math.BigDecimal;
 import java.math.BigInteger;
 import java.util.ArrayList;
+import java.util.Collection;
 import java.util.Iterator;
 import java.util.List;
 import java.util.NoSuchElementException;
@@ -42,9 +44,9 @@ import org.apache.commons.logging.impl.NoOpLog;
  * store any data.</p>
  * <p>If you want to write your own Configuration class then you should
  * implement only abstract methods from this class. A lot of functionality
- * needed by typical implementations of the <code>Configuration</conde>
+ * needed by typical implementations of the <code>Configuration</code>
  * interface is already provided by this base class. Following is a list of
- * feauters implemented here:
+ * features implemented here:
  * <ul><li>Data conversion support. The various data types required by the
  * <code>Configuration</code> interface are already handled by this base class.
  * A concrete sub class only needs to provide a generic <code>getProperty()</code>
@@ -54,7 +56,7 @@ import org.apache.commons.logging.impl.NoOpLog;
  * corresponding values.</li>
  * <li>Support for string lists. The values of properties to be added to this
  * configuration are checked whether they contain a list delimiter character. If
- * this is the case and if list splitting is enabled, the string is splitted and
+ * this is the case and if list splitting is enabled, the string is split and
  * multiple values are added for this property. (With the
  * <code>setListDelimiter()</code> method the delimiter character can be
  * specified; per default a comma is used. The
@@ -71,10 +73,9 @@ import org.apache.commons.logging.impl.NoOpLog;
  * </ul></p>
  *
  * @author <a href="mailto:ksh@scand.com">Konstantin Shaposhnikov </a>
- * @author <a href="mailto:oliver.heger@t-online.de">Oliver Heger </a>
+ * @author Oliver Heger
  * @author <a href="mailto:hps@intermeta.de">Henning P. Schmiedehausen </a>
- * @version $Id: AbstractConfiguration.java,v 1.29 2004/12/02 22:05:52 ebourg
- * Exp $
+ * @version $Id: AbstractConfiguration.java 589138 2007-10-27 15:41:13Z oheger $
  */
 public abstract class AbstractConfiguration extends EventSource implements Configuration
 {
@@ -115,6 +116,13 @@ public abstract class AbstractConfiguration extends EventSource implements Confi
     /** end token */
     protected static final String END_TOKEN = "}";
 
+    /**
+     * Constant for the disabled list delimiter. This character is passed to the
+     * list parsing methods if delimiter parsing is disabled. So this character
+     * should not occur in string property values.
+     */
+    private static final char DISABLED_DELIMITER = '\0';
+
     /** The default value for listDelimiter */
     private static char defaultListDelimiter = ',';
 
@@ -375,38 +383,43 @@ public abstract class AbstractConfiguration extends EventSource implements Confi
         });
     }
 
-    /**
-     * {@inheritDoc}
-     */
     public void addProperty(String key, Object value)
     {
         fireEvent(EVENT_ADD_PROPERTY, key, value, true);
-
-        if (!isDelimiterParsingDisabled())
-        {
-            Iterator it = PropertyConverter.toIterator(value, getListDelimiter());
-            while (it.hasNext())
-            {
-                addPropertyDirect(key, it.next());
-            }
-        }
-        else
-        {
-            addPropertyDirect(key, value);
-        }
-
+        addPropertyValues(key, value,
+                isDelimiterParsingDisabled() ? DISABLED_DELIMITER
+                        : getListDelimiter());
         fireEvent(EVENT_ADD_PROPERTY, key, value, false);
     }
 
     /**
      * Adds a key/value pair to the Configuration. Override this method to
-     * provide write acces to underlying Configuration store.
+     * provide write access to underlying Configuration store.
      *
      * @param key key to use for mapping
      * @param value object to store
      */
     protected abstract void addPropertyDirect(String key, Object value);
 
+    /**
+     * Adds the specified value for the given property. This method supports
+     * single values and containers (e.g. collections or arrays) as well. In the
+     * latter case, <code>addPropertyDirect()</code> will be called for each
+     * element.
+     *
+     * @param key the property key
+     * @param value the value object
+     * @param delimiter the list delimiter character
+     */
+    private void addPropertyValues(String key, Object value, char delimiter)
+    {
+        Iterator it = PropertyConverter.toIterator(value, delimiter);
+        while (it.hasNext())
+        {
+            addPropertyDirect(key, it.next());
+        }
+    }
+
     /**
      * interpolate key names to handle ${key} stuff
      *
@@ -453,27 +466,11 @@ public abstract class AbstractConfiguration extends EventSource implements Confi
         return base; // just a dummy implementation
     }
 
-    /**
-     * {@inheritDoc}
-     */
     public Configuration subset(String prefix)
     {
         return new SubsetConfiguration(this, prefix, ".");
     }
 
-    /**
-     * {@inheritDoc}
-     */
-    public abstract boolean isEmpty();
-
-    /**
-     * {@inheritDoc}
-     */
-    public abstract boolean containsKey(String key);
-
-    /**
-     * {@inheritDoc}
-     */
     public void setProperty(String key, Object value)
     {
         fireEvent(EVENT_SET_PROPERTY, key, value, true);
@@ -517,24 +514,38 @@ public abstract class AbstractConfiguration extends EventSource implements Confi
         // override in sub classes
     }
 
-    /**
-     * {@inheritDoc}
-     */
     public void clear()
     {
         fireEvent(EVENT_CLEAR, null, null, true);
         setDetailEvents(false);
+        boolean useIterator = true;
         try
         {
             Iterator it = getKeys();
             while (it.hasNext())
             {
                 String key = (String) it.next();
-                it.remove();
-
-                if (containsKey(key))
+                if (useIterator)
                 {
-                    // workaround for Iterators that do not remove the property on calling remove()
+                    try
+                    {
+                        it.remove();
+                    }
+                    catch (UnsupportedOperationException usoex)
+                    {
+                        useIterator = false;
+                    }
+                }
+
+                if (useIterator && containsKey(key))
+                {
+                    useIterator = false;
+                }
+
+                if (!useIterator)
+                {
+                    // workaround for Iterators that do not remove the property
+                    // on calling remove() or do not support remove() at all
                     clearProperty(key);
                 }
             }
@@ -546,14 +557,6 @@ public abstract class AbstractConfiguration extends EventSource implements Confi
         fireEvent(EVENT_CLEAR, null, null, false);
     }
 
-    /**
-     * {@inheritDoc}
-     */
-    public abstract Iterator getKeys();
-
-    /**
-     * {@inheritDoc}
-     */
     public Iterator getKeys(final String prefix)
     {
         return new FilterIterator(getKeys(), new Predicate()
@@ -566,9 +569,6 @@ public abstract class AbstractConfiguration extends EventSource implements Confi
         });
     }
 
-    /**
-     * {@inheritDoc}
-     */
     public Properties getProperties(String key)
     {
         return getProperties(key, null);
@@ -683,9 +683,6 @@ public abstract class AbstractConfiguration extends EventSource implements Confi
         }
     }
 
-    /**
-     * {@inheritDoc}
-     */
     public byte getByte(String key)
     {
         Byte b = getByte(key, null);
@@ -699,17 +696,11 @@ public abstract class AbstractConfiguration extends EventSource implements Confi
         }
     }
 
-    /**
-     * {@inheritDoc}
-     */
     public byte getByte(String key, byte defaultValue)
     {
         return getByte(key, new Byte(defaultValue)).byteValue();
     }
 
-    /**
-     * {@inheritDoc}
-     */
     public Byte getByte(String key, Byte defaultValue)
     {
         Object value = resolveContainerStore(key);
@@ -731,9 +722,6 @@ public abstract class AbstractConfiguration extends EventSource implements Confi
         }
     }
 
-    /**
-     * {@inheritDoc}
-     */
     public double getDouble(String key)
     {
         Double d = getDouble(key, null);
@@ -747,17 +735,11 @@ public abstract class AbstractConfiguration extends EventSource implements Confi
         }
     }
 
-    /**
-     * {@inheritDoc}
-     */
     public double getDouble(String key, double defaultValue)
     {
         return getDouble(key, new Double(defaultValue)).doubleValue();
     }
 
-    /**
-     * {@inheritDoc}
-     */
     public Double getDouble(String key, Double defaultValue)
     {
         Object value = resolveContainerStore(key);
@@ -779,9 +761,6 @@ public abstract class AbstractConfiguration extends EventSource implements Confi
         }
     }
 
-    /**
-     * {@inheritDoc}
-     */
     public float getFloat(String key)
     {
         Float f = getFloat(key, null);
@@ -795,17 +774,11 @@ public abstract class AbstractConfiguration extends EventSource implements Confi
         }
     }
 
-    /**
-     * {@inheritDoc}
-     */
     public float getFloat(String key, float defaultValue)
     {
         return getFloat(key, new Float(defaultValue)).floatValue();
     }
 
-    /**
-     * {@inheritDoc}
-     */
     public Float getFloat(String key, Float defaultValue)
     {
         Object value = resolveContainerStore(key);
@@ -827,9 +800,6 @@ public abstract class AbstractConfiguration extends EventSource implements Confi
         }
     }
 
-    /**
-     * {@inheritDoc}
-     */
     public int getInt(String key)
     {
         Integer i = getInteger(key, null);
@@ -843,9 +813,6 @@ public abstract class AbstractConfiguration extends EventSource implements Confi
         }
     }
 
-    /**
-     * {@inheritDoc}
-     */
     public int getInt(String key, int defaultValue)
     {
         Integer i = getInteger(key, null);
@@ -858,9 +825,6 @@ public abstract class AbstractConfiguration extends EventSource implements Confi
         return i.intValue();
     }
 
-    /**
-     * {@inheritDoc}
-     */
     public Integer getInteger(String key, Integer defaultValue)
     {
         Object value = resolveContainerStore(key);
@@ -882,9 +846,6 @@ public abstract class AbstractConfiguration extends EventSource implements Confi
         }
     }
 
-    /**
-     * {@inheritDoc}
-     */
     public long getLong(String key)
     {
         Long l = getLong(key, null);
@@ -898,17 +859,11 @@ public abstract class AbstractConfiguration extends EventSource implements Confi
         }
     }
 
-    /**
-     * {@inheritDoc}
-     */
     public long getLong(String key, long defaultValue)
     {
         return getLong(key, new Long(defaultValue)).longValue();
     }
 
-    /**
-     * {@inheritDoc}
-     */
     public Long getLong(String key, Long defaultValue)
     {
         Object value = resolveContainerStore(key);
@@ -930,9 +885,6 @@ public abstract class AbstractConfiguration extends EventSource implements Confi
         }
     }
 
-    /**
-     * {@inheritDoc}
-     */
     public short getShort(String key)
     {
         Short s = getShort(key, null);
@@ -946,17 +898,11 @@ public abstract class AbstractConfiguration extends EventSource implements Confi
         }
     }
 
-    /**
-     * {@inheritDoc}
-     */
     public short getShort(String key, short defaultValue)
     {
         return getShort(key, new Short(defaultValue)).shortValue();
     }
 
-    /**
-     * {@inheritDoc}
-     */
     public Short getShort(String key, Short defaultValue)
     {
         Object value = resolveContainerStore(key);
@@ -999,9 +945,6 @@ public abstract class AbstractConfiguration extends EventSource implements Confi
         }
     }
 
-    /**
-     * {@inheritDoc}
-     */
     public BigDecimal getBigDecimal(String key, BigDecimal defaultValue)
     {
         Object value = resolveContainerStore(key);
@@ -1044,9 +987,6 @@ public abstract class AbstractConfiguration extends EventSource implements Confi
         }
     }
 
-    /**
-     * {@inheritDoc}
-     */
     public BigInteger getBigInteger(String key, BigInteger defaultValue)
     {
         Object value = resolveContainerStore(key);
@@ -1089,9 +1029,6 @@ public abstract class AbstractConfiguration extends EventSource implements Confi
         }
     }
 
-    /**
-     * {@inheritDoc}
-     */
     public String getString(String key, String defaultValue)
     {
         Object value = resolveContainerStore(key);
@@ -1171,9 +1108,6 @@ public abstract class AbstractConfiguration extends EventSource implements Confi
         return getList(key, new ArrayList());
     }
 
-    /**
-     * {@inheritDoc}
-     */
     public List getList(String key, List defaultValue)
     {
         Object value = getProperty(key);
@@ -1195,7 +1129,6 @@ public abstract class AbstractConfiguration extends EventSource implements Confi
             {
                 list.add(interpolate(it.next()));
             }
-
         }
         else if (value == null)
         {
@@ -1211,65 +1144,132 @@ public abstract class AbstractConfiguration extends EventSource implements Confi
 
     /**
      * Returns an object from the store described by the key. If the value is a
-     * List object, replace it with the first object in the list.
+     * Collection object, replace it with the first object in the collection.
      *
      * @param key The property key.
      *
-     * @return value Value, transparently resolving a possible List dependency.
+     * @return value Value, transparently resolving a possible collection dependency.
      */
     protected Object resolveContainerStore(String key)
     {
         Object value = getProperty(key);
         if (value != null)
         {
-            if (value instanceof List)
+            if (value instanceof Collection)
             {
-                List list = (List) value;
-                value = list.isEmpty() ? null : list.get(0);
+                Collection collection = (Collection) value;
+                value = collection.isEmpty() ? null : collection.iterator().next();
             }
-            else if (value instanceof Object[])
+            else if (value.getClass().isArray() && Array.getLength(value) > 0)
             {
-                Object[] array = (Object[]) value;
-                value = array.length == 0 ? null : array[0];
-            }
-            else if (value instanceof boolean[])
-            {
-                boolean[] array = (boolean[]) value;
-                value = array.length == 0 ? null : array[0] ? Boolean.TRUE : Boolean.FALSE;
-            }
-            else if (value instanceof byte[])
-            {
-                byte[] array = (byte[]) value;
-                value = array.length == 0 ? null : new Byte(array[0]);
-            }
-            else if (value instanceof short[])
-            {
-                short[] array = (short[]) value;
-                value = array.length == 0 ? null : new Short(array[0]);
-            }
-            else if (value instanceof int[])
-            {
-                int[] array = (int[]) value;
-                value = array.length == 0 ? null : new Integer(array[0]);
-            }
-            else if (value instanceof long[])
-            {
-                long[] array = (long[]) value;
-                value = array.length == 0 ? null : new Long(array[0]);
-            }
-            else if (value instanceof float[])
-            {
-                float[] array = (float[]) value;
-                value = array.length == 0 ? null : new Float(array[0]);
-            }
-            else if (value instanceof double[])
-            {
-                double[] array = (double[]) value;
-                value = array.length == 0 ? null : new Double(array[0]);
+                value = Array.get(value, 0);
             }
         }
 
         return value;
     }
 
+    /**
+     * Copies the content of the specified configuration into this
+     * configuration. If the specified configuration contains a key that is also
+     * present in this configuration, the value of this key will be replaced by
+     * the new value. <em>Note:</em> This method won't work well when copying
+     * hierarchical configurations because it is not able to copy information
+     * about the properties' structure (i.e. the parent-child-relationships will
+     * get lost). So when dealing with hierarchical configuration objects their
+     * <code>{@link HierarchicalConfiguration#clone() clone()}</code> methods
+     * should be used.
+     *
+     * @param c the configuration to copy (can be <b>null</b>, then this
+     * operation will have no effect)
+     * @since 1.5
+     */
+    public void copy(Configuration c)
+    {
+        if (c != null)
+        {
+            for (Iterator it = c.getKeys(); it.hasNext();)
+            {
+                String key = (String) it.next();
+                Object value = c.getProperty(key);
+                fireEvent(EVENT_SET_PROPERTY, key, value, true);
+                setDetailEvents(false);
+                try
+                {
+                    clearProperty(key);
+                    addPropertyValues(key, value, DISABLED_DELIMITER);
+                }
+                finally
+                {
+                    setDetailEvents(true);
+                }
+                fireEvent(EVENT_SET_PROPERTY, key, value, false);
+            }
+        }
+    }
+
+    /**
+     * Appends the content of the specified configuration to this configuration.
+     * The values of all properties contained in the specified configuration
+     * will be appended to this configuration. So if a property is already
+     * present in this configuration, its new value will be a union of the
+     * values in both configurations. <em>Note:</em> This method won't work
+     * well when appending hierarchical configurations because it is not able to
+     * copy information about the properties' structure (i.e. the
+     * parent-child-relationships will get lost). So when dealing with
+     * hierarchical configuration objects their
+     * <code>{@link HierarchicalConfiguration#clone() clone()}</code> methods
+     * should be used.
+     *
+     * @param c the configuration to be appended (can be <b>null</b>, then this
+     * operation will have no effect)
+     * @since 1.5
+     */
+    public void append(Configuration c)
+    {
+        if (c != null)
+        {
+            for (Iterator it = c.getKeys(); it.hasNext();)
+            {
+                String key = (String) it.next();
+                Object value = c.getProperty(key);
+                fireEvent(EVENT_ADD_PROPERTY, key, value, true);
+                addPropertyValues(key, value, DISABLED_DELIMITER);
+                fireEvent(EVENT_ADD_PROPERTY, key, value, false);
+            }
+        }
+    }
+
+    /**
+     * Returns a configuration with the same content as this configuration, but
+     * with all variables replaced by their actual values. This method tries to
+     * clone the configuration and then perform interpolation on all properties.
+     * So property values of the form <code>${var}</code> will be resolved as
+     * far as possible (if a variable cannot be resolved, it remains unchanged).
+     * This operation is useful if the content of a configuration is to be
+     * exported or processed by an external component that does not support
+     * variable interpolation.
+     *
+     * @return a configuration with all variables interpolated
+     * @throws ConfigurationRuntimeException if this configuration cannot be
+     * cloned
+     * @since 1.5
+     */
+    public Configuration interpolatedConfiguration()
+    {
+        // first clone this configuration
+        AbstractConfiguration c = (AbstractConfiguration) ConfigurationUtils
+                .cloneConfiguration(this);
+
+        // now perform interpolation
+        c.setDelimiterParsingDisabled(true);
+        for (Iterator it = getKeys(); it.hasNext();)
+        {
+            String key = (String) it.next();
+            c.setProperty(key, getList(key));
+        }
+
+        c.setDelimiterParsingDisabled(isDelimiterParsingDisabled());
+        return c;
+    }
 }
