@@ -27,8 +27,10 @@ import java.io.OutputStreamWriter;
 import java.io.Reader;
 import java.io.UnsupportedEncodingException;
 import java.io.Writer;
+import java.net.HttpURLConnection;
 import java.net.MalformedURLException;
 import java.net.URL;
+import java.net.URLConnection;
 import java.util.Iterator;
 
 import org.apache.commons.configuration.reloading.InvariantReloadingStrategy;
@@ -74,7 +76,7 @@ import org.apache.commons.logging.LogFactory;
  * in <code>java.util.Properties</code>.</p>
  *
  * @author Emmanuel Bourg
- * @version $Revision: 497574 $, $Date: 2007-01-18 22:02:55 +0100 (Do, 18 Jan 2007) $
+ * @version $Revision: 545904 $, $Date: 2007-06-10 18:10:17 +0200 (So, 10 Jun 2007) $
  * @since 1.0-rc2
  */
 public abstract class AbstractFileConfiguration extends BaseConfiguration implements FileConfiguration
@@ -409,7 +411,7 @@ public abstract class AbstractFileConfiguration extends BaseConfiguration implem
     }
 
     /**
-     * Save the configuration to the specified URL if it's a file URL.
+     * Save the configuration to the specified URL.
      * This doesn't change the source of the configuration, use setURL()
      * if you need it.
      *
@@ -419,6 +421,8 @@ public abstract class AbstractFileConfiguration extends BaseConfiguration implem
      */
     public void save(URL url) throws ConfigurationException
     {
+        // file URLs have to be converted to Files since FileURLConnection is
+        // read only (http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=4191800)
         File file = ConfigurationUtils.fileFromURL(url);
         if (file != null)
         {
@@ -426,7 +430,41 @@ public abstract class AbstractFileConfiguration extends BaseConfiguration implem
         }
         else
         {
-            throw new ConfigurationException("Could not save to URL " + url);
+            // for non file URLs save through an URLConnection
+            OutputStream out = null;
+            try
+            {
+                URLConnection connection = url.openConnection();
+                connection.setDoOutput(true);
+
+                // use the PUT method for http URLs
+                if (connection instanceof HttpURLConnection)
+                {
+                    HttpURLConnection conn = (HttpURLConnection) connection;
+                    conn.setRequestMethod("PUT");
+                }
+
+                out = connection.getOutputStream();
+                save(out);
+
+                // check the response code for http URLs and throw an exception if an error occured
+                if (connection instanceof HttpURLConnection)
+                {
+                    HttpURLConnection conn = (HttpURLConnection) connection;
+                    if (conn.getResponseCode() >= HttpURLConnection.HTTP_BAD_REQUEST)
+                    {
+                        throw new IOException("HTTP Error " + conn.getResponseCode() + " " + conn.getResponseMessage());
+                    }
+                }
+            }
+            catch (IOException e)
+            {
+                throw new ConfigurationException("Could not save to URL " + url, e);
+            }
+            finally
+            {
+                closeSilent(out);
+            }
         }
     }
 
@@ -456,18 +494,7 @@ public abstract class AbstractFileConfiguration extends BaseConfiguration implem
         }
         finally
         {
-            // close the output stream
-            try
-            {
-                if (out != null)
-                {
-                    out.close();
-                }
-            }
-            catch (IOException e)
-            {
-                getLogger().warn("Could not close output stream", e);
-            }
+            closeSilent(out);
         }
     }
 
@@ -585,20 +612,17 @@ public abstract class AbstractFileConfiguration extends BaseConfiguration implem
      */
     public File getFile()
     {
-        if (getFileName() == null)
+        if (getFileName() == null && sourceURL == null)
         {
             return null;
         }
+        else if (sourceURL != null)
+        {
+            return ConfigurationUtils.fileFromURL(sourceURL);
+        }
         else
         {
-            if (sourceURL != null)
-            {
-                return ConfigurationUtils.fileFromURL(sourceURL);
-            }
-            else
-            {
-                return ConfigurationUtils.getFile(getBasePath(), getFileName());
-            }
+            return ConfigurationUtils.getFile(getBasePath(), getFileName());
         }
     }
 
@@ -647,8 +671,7 @@ public abstract class AbstractFileConfiguration extends BaseConfiguration implem
             {
                 try
                 {
-                    path = ConfigurationUtils.getURL(getBasePath(),
-                            getFileName()).getPath();
+                    path = ConfigurationUtils.getURL(getBasePath(), getFileName()).getPath();
                 }
                 catch (MalformedURLException e)
                 {
@@ -804,6 +827,8 @@ public abstract class AbstractFileConfiguration extends BaseConfiguration implem
                         }
                         fireEvent(EVENT_RELOAD, null, getURL(), true);
                         setDetailEvents(false);
+                        boolean autoSaveBak = this.isAutoSave(); // save the current state
+                        this.setAutoSave(false); // deactivate autoSave to prevent information loss
                         try
                         {
                             clear();
@@ -811,6 +836,7 @@ public abstract class AbstractFileConfiguration extends BaseConfiguration implem
                         }
                         finally
                         {
+                            this.setAutoSave(autoSaveBak); // set autoSave to previous value
                             setDetailEvents(true);
                         }
                         fireEvent(EVENT_RELOAD, null, getURL(), false);
@@ -877,8 +903,7 @@ public abstract class AbstractFileConfiguration extends BaseConfiguration implem
      * @param propValue the value of the property
      * @param before the before update flag
      */
-    protected void fireEvent(int type, String propName, Object propValue,
-            boolean before)
+    protected void fireEvent(int type, String propName, Object propValue, boolean before)
     {
         enterNoReload();
         try
@@ -959,8 +984,7 @@ public abstract class AbstractFileConfiguration extends BaseConfiguration implem
      */
     public Object clone()
     {
-        AbstractFileConfiguration copy = (AbstractFileConfiguration) super
-                .clone();
+        AbstractFileConfiguration copy = (AbstractFileConfiguration) super.clone();
         copy.setBasePath(null);
         copy.setFileName(null);
         copy.initReloadingStrategy();
@@ -974,4 +998,26 @@ public abstract class AbstractFileConfiguration extends BaseConfiguration implem
     {
         setReloadingStrategy(new InvariantReloadingStrategy());
     }
+
+    /**
+     * A helper method for closing an output stream. Occurring exceptions will
+     * be ignored.
+     *
+     * @param out the output stream to be closed (may be <b>null</b>)
+     * @since 1.5
+     */
+    private void closeSilent(OutputStream out)
+    {
+        try
+        {
+            if (out != null)
+            {
+                out.close();
+            }
+        }
+        catch (IOException e)
+        {
+            getLogger().warn("Could not close output stream", e);
+        }
+    }
 }
