@@ -33,12 +33,52 @@ import org.apache.commons.collections.CollectionUtils;
 import org.apache.commons.logging.LogFactory;
 
 /**
- * Configuration stored in a database.
+ * Configuration stored in a database. The properties are retrieved from a
+ * table containing at least one column for the keys, and one column for the
+ * values. It's possible to store several configurations in the same table by
+ * adding a column containing the name of the configuration. The name of the
+ * table and the columns is specified in the constructor.
+ *
+ * <h4>Example 1 - One configuration per table</h4>
+ *
+ * <pre>
+ * CREATE TABLE myconfig (
+ *     `key`   VARCHAR NOT NULL PRIMARY KEY,
+ *     `value` VARCHAR
+ * );
+ *
+ * INSERT INTO myconfig (key, value) VALUES ('foo', 'bar');
+ *
+ *
+ * Configuration config = new DatabaseConfiguration(datasource, "myconfig", "key", "value");
+ * String value = config.getString("foo");
+ * </pre>
+ *
+ * <h4>Example 2 - Multiple configurations per table</h4>
+ *
+ * <pre>
+ * CREATE TABLE myconfigs (
+ *     `name`  VARCHAR NOT NULL,
+ *     `key`   VARCHAR NOT NULL,
+ *     `value` VARCHAR,
+ *     CONSTRAINT sys_pk_myconfigs PRIMARY KEY (`name`, `key`)
+ * );
+ *
+ * INSERT INTO myconfigs (name, key, value) VALUES ('config1', 'key1', 'value1');
+ * INSERT INTO myconfigs (name, key, value) VALUES ('config2', 'key2', 'value2');
+ *
+ *
+ * Configuration config1 = new DatabaseConfiguration(datasource, "myconfigs", "name", "key", "value", "config1");
+ * String value1 = conf.getString("key1");
+ *
+ * Configuration config2 = new DatabaseConfiguration(datasource, "myconfigs", "name", "key", "value", "config2");
+ * String value2 = conf.getString("key2");
+ * </pre>
  *
  * @since 1.0
  *
- * @author Emmanuel Bourg
- * @version $Revision: 514234 $, $Date: 2007-03-03 21:18:14 +0100 (Sa, 03 Mrz 2007) $
+ * @author <a href="mailto:ebourg@apache.org">Emmanuel Bourg</a>
+ * @version $Revision: 589380 $, $Date: 2007-10-28 17:37:35 +0100 (So, 28 Okt 2007) $
  */
 public class DatabaseConfiguration extends AbstractConfiguration
 {
@@ -139,16 +179,15 @@ public class DatabaseConfiguration extends AbstractConfiguration
             List results = new ArrayList();
             while (rs.next())
             {
-                Object val = rs.getObject(valueColumn);
+                Object value = rs.getObject(valueColumn);
                 if (isDelimiterParsingDisabled())
                 {
-                    results.add(val);
+                    results.add(value);
                 }
                 else
                 {
                     // Split value if it containts the list delimiter
-                    CollectionUtils.addAll(results, PropertyConverter
-                            .toIterator(val, getListDelimiter()));
+                    CollectionUtils.addAll(results, PropertyConverter.toIterator(value, getListDelimiter()));
                 }
             }
 
@@ -163,7 +202,7 @@ public class DatabaseConfiguration extends AbstractConfiguration
         }
         finally
         {
-            closeQuietly(conn, pstmt);
+            close(conn, pstmt);
         }
 
         return result;
@@ -218,7 +257,7 @@ public class DatabaseConfiguration extends AbstractConfiguration
         finally
         {
             // clean up
-            closeQuietly(conn, pstmt);
+            close(conn, pstmt);
         }
     }
 
@@ -299,7 +338,7 @@ public class DatabaseConfiguration extends AbstractConfiguration
         finally
         {
             // clean up
-            closeQuietly(conn, pstmt);
+            close(conn, pstmt);
         }
 
         return empty;
@@ -352,7 +391,7 @@ public class DatabaseConfiguration extends AbstractConfiguration
         finally
         {
             // clean up
-            closeQuietly(conn, pstmt);
+            close(conn, pstmt);
         }
 
         return found;
@@ -400,7 +439,7 @@ public class DatabaseConfiguration extends AbstractConfiguration
         finally
         {
             // clean up
-            closeQuietly(conn, pstmt);
+            close(conn, pstmt);
         }
     }
 
@@ -443,7 +482,7 @@ public class DatabaseConfiguration extends AbstractConfiguration
         finally
         {
             // clean up
-            closeQuietly(conn, pstmt);
+            close(conn, pstmt);
         }
     }
 
@@ -496,7 +535,7 @@ public class DatabaseConfiguration extends AbstractConfiguration
         finally
         {
             // clean up
-            closeQuietly(conn, pstmt);
+            close(conn, pstmt);
         }
 
         return keys.iterator();
@@ -521,6 +560,8 @@ public class DatabaseConfiguration extends AbstractConfiguration
      * @return the <code>Connection</code> object to be used
      * @throws SQLException if an error occurs
      * @since 1.4
+     * @deprecated Use a custom data source to change the connection used by the
+     * class. To be removed in Commons Configuration 2.0
      */
     protected Connection getConnection() throws SQLException
     {
@@ -534,7 +575,7 @@ public class DatabaseConfiguration extends AbstractConfiguration
      * @param conn The database connection to close
      * @param stmt The statement to close
      */
-    private void closeQuietly(Connection conn, Statement stmt)
+    private void close(Connection conn, Statement stmt)
     {
         try
         {
@@ -542,6 +583,14 @@ public class DatabaseConfiguration extends AbstractConfiguration
             {
                 stmt.close();
             }
+        }
+        catch (SQLException e)
+        {
+            getLogger().error("An error occured on closing the statement", e);
+        }
+
+        try
+        {
             if (conn != null)
             {
                 conn.close();
@@ -549,7 +598,7 @@ public class DatabaseConfiguration extends AbstractConfiguration
         }
         catch (SQLException e)
         {
-            getLogger().error(e.getMessage(), e);
+            getLogger().error("An error occured on closing the connection", e);
         }
     }
 }
