@@ -28,7 +28,10 @@ import java.util.Stack;
 
 import org.apache.commons.collections.set.ListOrderedSet;
 import org.apache.commons.collections.iterators.SingletonIterator;
+import org.apache.commons.configuration.event.ConfigurationEvent;
+import org.apache.commons.configuration.event.ConfigurationListener;
 import org.apache.commons.configuration.tree.ConfigurationNode;
+import org.apache.commons.configuration.tree.ConfigurationNodeVisitor;
 import org.apache.commons.configuration.tree.ConfigurationNodeVisitorAdapter;
 import org.apache.commons.configuration.tree.DefaultConfigurationNode;
 import org.apache.commons.configuration.tree.DefaultExpressionEngine;
@@ -86,7 +89,7 @@ import org.apache.commons.lang.StringUtils;
  * element if there are multiple hits.</p><p>For instance the key
  * <code>tables.table(0).name</code> can be used to find out the name of the
  * first table. In opposite <code>tables.table.name</code> would return a
- * collection with the names of all available tables. Similarily the key
+ * collection with the names of all available tables. Similarly the key
  * <code>tables.table(1).fields.field.name</code> returns a collection with
  * the names of all fields of the second table. If another index is added after
  * the <code>field</code> element, a single field can be accessed:
@@ -94,26 +97,62 @@ import org.apache.commons.lang.StringUtils;
  * <code>getMaxIndex()</code> method that returns the maximum allowed index
  * that can be added to a given property key. This method can be used to iterate
  * over all values defined for a certain property.</p>
+ * <p>Since the 1.3 release of <em>Commons Configuration</em> hierarchical
+ * configurations support an <em>expression engine</em>. This expression engine
+ * is responsible for evaluating the passed in configuration keys and map them
+ * to the stored properties. The examples above are valid for the default
+ * expression engine, which is used when a new <code>HierarchicalConfiguration</code>
+ * instance is created. With the <code>setExpressionEngine()</code> method a
+ * different expression engine can be set. For instance with
+ * <code>{@link org.apache.commons.configuration.tree.xpath.XPathExpressionEngine}</code>
+ * there is an expression engine available that supports configuration keys in
+ * XPATH syntax.</p>
+ * <p>In addition to the events common for all configuration classes hierarchical
+ * configurations support some more events that correspond to some specific
+ * methods and features:
+ * <dl><dt><em>EVENT_ADD_NODES</em></dt><dd>The <code>addNodes()</code> method
+ * was called; the event object contains the key, to which the nodes were added,
+ * and a collection with the new nodes as value.</dd>
+ * <dt><em>EVENT_CLEAR_TREE</em></dt><dd>The <code>clearTree()</code> method was
+ * called; the event object stores the key of the removed sub tree.</dd>
+ * <dt><em>EVENT_SUBNODE_CHANGED</em></dt><dd>A <code>SubnodeConfiguration</code>
+ * that was created from this configuration has been changed. The value property
+ * of the event object contains the original event object as it was sent by the
+ * subnode configuration.</dd></dl></p>
+ * <p><em>Note:</em>Configuration objects of this type can be read concurrently
+ * by multiple threads. However if one of these threads modifies the object,
+ * synchronization has to be performed manually.</p>
  *
- * @author <a href="mailto:oliver.heger@t-online.de">Oliver Heger </a>
- * @version $Id: HierarchicalConfiguration.java,v 1.14 2004/12/02 22:05:52
- * ebourg Exp $
+ * @author Oliver Heger
+ * @version $Id: HierarchicalConfiguration.java 593319 2007-11-08 21:14:21Z oheger $
  */
 public class HierarchicalConfiguration extends AbstractConfiguration implements Serializable, Cloneable
 {
-    /** Constant for the clear tree event.*/
+    /**
+     * Constant for the clear tree event.
+     * @since 1.3
+     */
     public static final int EVENT_CLEAR_TREE = 10;
 
-    /** Constant for the add nodes event.*/
+    /**
+     * Constant for the add nodes event.
+     * @since 1.3
+     */
     public static final int EVENT_ADD_NODES = 11;
 
+    /**
+     * Constant for the subnode configuration modified event.
+     * @since 1.5
+     */
+    public static final int EVENT_SUBNODE_CHANGED = 12;
+
     /**
      * The serial version UID.
      */
     private static final long serialVersionUID = 3373812230395363192L;
 
     /** Stores the default expression engine to be used for new objects.*/
-    private static ExpressionEngine defaultExpressionEngine = new DefaultExpressionEngine();
+    private static ExpressionEngine defaultExpressionEngine;
 
     /** Stores the root node of this configuration. This field is required for
      * backwards compatibility only.
@@ -124,7 +163,7 @@ public class HierarchicalConfiguration extends AbstractConfiguration implements
     private ConfigurationNode rootNode;
 
     /** Stores the expression engine for this instance.*/
-    private ExpressionEngine expressionEngine;
+    private transient ExpressionEngine expressionEngine;
 
     /**
      * Creates a new instance of <code>HierarchicalConfiguration</code>.
@@ -220,8 +259,12 @@ public class HierarchicalConfiguration extends AbstractConfiguration implements
      * @return the default expression engine
      * @since 1.3
      */
-    public static ExpressionEngine getDefaultExpressionEngine()
+    public static synchronized ExpressionEngine getDefaultExpressionEngine()
     {
+        if (defaultExpressionEngine == null)
+        {
+            defaultExpressionEngine = new DefaultExpressionEngine();
+        }
         return defaultExpressionEngine;
     }
 
@@ -234,7 +277,7 @@ public class HierarchicalConfiguration extends AbstractConfiguration implements
      * @param engine the new default expression engine
      * @since 1.3
      */
-    public static void setDefaultExpressionEngine(ExpressionEngine engine)
+    public static synchronized void setDefaultExpressionEngine(ExpressionEngine engine)
     {
         if (engine == null)
         {
@@ -331,9 +374,15 @@ public class HierarchicalConfiguration extends AbstractConfiguration implements
      * instead of a single property a whole collection of nodes can be added -
      * and thus complete configuration sub trees. E.g. with this method it is
      * possible to add parts of another <code>HierarchicalConfiguration</code>
-     * object to this object. If the passed in key refers to an existing and
-     * unique node, the new nodes are added to this node. Otherwise a new node
-     * will be created at the specified position in the hierarchy.
+     * object to this object. (However be aware that a
+     * <code>ConfigurationNode</code> object can only belong to a single
+     * configuration. So if nodes from one configuration are directly added to
+     * another one using this method, the structure of the source configuration
+     * will be broken. In this case you should clone the nodes to be added
+     * before calling <code>addNodes()</code>.) If the passed in key refers to
+     * an existing and unique node, the new nodes are added to this node.
+     * Otherwise a new node will be created at the specified position in the
+     * hierarchy.
      *
      * @param key the key where the nodes are to be added; can be <b>null </b>,
      * then they are added to the root node
@@ -367,6 +416,17 @@ public class HierarchicalConfiguration extends AbstractConfiguration implements
             throw new IllegalArgumentException(
                     "Cannot add nodes to an attribute node!");
         }
+
+        // a visitor to ensure that the nodes' references are cleared; this is
+        // necessary if the nodes are moved from another configuration
+        ConfigurationNodeVisitor clearRefVisitor = new ConfigurationNodeVisitorAdapter()
+        {
+            public void visitBeforeChildren(ConfigurationNode node)
+            {
+                node.setReference(null);
+            }
+        };
+
         for (Iterator it = nodes.iterator(); it.hasNext();)
         {
             ConfigurationNode child = (ConfigurationNode) it.next();
@@ -378,6 +438,7 @@ public class HierarchicalConfiguration extends AbstractConfiguration implements
             {
                 parent.addChild(child);
             }
+            child.visit(clearRefVisitor);
         }
         fireEvent(EVENT_ADD_NODES, key, nodes, false);
     }
@@ -397,7 +458,16 @@ public class HierarchicalConfiguration extends AbstractConfiguration implements
      * Creates a new <code>Configuration</code> object containing all keys
      * that start with the specified prefix. This implementation will return a
      * <code>HierarchicalConfiguration</code> object so that the structure of
-     * the keys will be saved.
+     * the keys will be saved. The nodes selected by the prefix (it is possible
+     * that multiple nodes are selected) are mapped to the root node of the
+     * returned configuration, i.e. their children and attributes will become
+     * children and attributes of the new root node. However a value of the root
+     * node is only set if exactly one of the selected nodes contain a value (if
+     * multiple nodes have a value, there is simply no way to decide how these
+     * values are merged together). Note that the returned
+     * <code>Configuration</code> object is not connected to its source
+     * configuration: updates on the source configuration are not reflected in
+     * the subset and vice versa.
      *
      * @param prefix the prefix of the keys for the subset
      * @return a new configuration object representing the selected subset
@@ -421,21 +491,37 @@ public class HierarchicalConfiguration extends AbstractConfiguration implements
         };
         CloneVisitor visitor = new CloneVisitor();
 
+        // Initialize the new root node
+        Object value = null;
+        int valueCount = 0;
         for (Iterator it = nodes.iterator(); it.hasNext();)
         {
             ConfigurationNode nd = (ConfigurationNode) it.next();
+            if (nd.getValue() != null)
+            {
+                value = nd.getValue();
+                valueCount++;
+            }
             nd.visit(visitor);
 
-            for (Iterator it2 = visitor.getClone().getChildren().iterator(); it2.hasNext();)
+            for (Iterator it2 = visitor.getClone().getChildren().iterator(); it2
+                    .hasNext();)
             {
                 result.getRootNode().addChild((ConfigurationNode) it2.next());
             }
-            for (Iterator it2 = visitor.getClone().getAttributes().iterator(); it2.hasNext();)
+            for (Iterator it2 = visitor.getClone().getAttributes().iterator(); it2
+                    .hasNext();)
             {
-                result.getRootNode().addAttribute((ConfigurationNode) it2.next());
+                result.getRootNode().addAttribute(
+                        (ConfigurationNode) it2.next());
             }
         }
 
+        // Determine the value of the new root
+        if (valueCount == 1)
+        {
+            result.getRootNode().setValue(value);
+        }
         return (result.isEmpty()) ? new HierarchicalConfiguration() : result;
     }
 
@@ -458,6 +544,52 @@ public class HierarchicalConfiguration extends AbstractConfiguration implements
      * <code>SubnodeConfiguration</code> class to obtain further information
      * about subnode configurations and when they should be used.
      * </p>
+     * <p>
+     * With the <code>supportUpdate</code> flag the behavior of the returned
+     * <code>SubnodeConfiguration</code> regarding updates of its parent
+     * configuration can be determined. A subnode configuration operates on the
+     * same nodes as its parent, so changes at one configuration are normally
+     * directly visible for the other configuration. There are however changes
+     * of the parent configuration, which are not recognized by the subnode
+     * configuration per default. An example for this is a reload operation (for
+     * file-based configurations): Here the complete node set of the parent
+     * configuration is replaced, but the subnode configuration still references
+     * the old nodes. If such changes should be detected by the subnode
+     * configuration, the <code>supportUpdates</code> flag must be set to
+     * <b>true</b>. This causes the subnode configuration to reevaluate the key
+     * used for its creation each time it is accessed. This guarantees that the
+     * subnode configuration always stays in sync with its key, even if the
+     * parent configuration's data significantly changes. If such a change
+     * makes the key invalid - because it now no longer points to exactly one
+     * node -, the subnode configuration is not reconstructed, but keeps its
+     * old data. It is then quasi detached from its parent.
+     * </p>
+     *
+     * @param key the key that selects the sub tree
+     * @param supportUpdates a flag whether the returned subnode configuration
+     * should be able to handle updates of its parent
+     * @return a hierarchical configuration that contains this sub tree
+     * @see SubnodeConfiguration
+     * @since 1.5
+     */
+    public SubnodeConfiguration configurationAt(String key,
+            boolean supportUpdates)
+    {
+        List nodes = fetchNodeList(key);
+        if (nodes.size() != 1)
+        {
+            throw new IllegalArgumentException(
+                    "Passed in key must select exactly one node: " + key);
+        }
+        return supportUpdates ? createSubnodeConfiguration(
+                (ConfigurationNode) nodes.get(0), key)
+                : createSubnodeConfiguration((ConfigurationNode) nodes.get(0));
+    }
+
+    /**
+     * Returns a hierarchical subnode configuration for the node specified by
+     * the given key. This is a short form for <code>configurationAt(key,
+     * <b>false</b>)</code>.
      *
      * @param key the key that selects the sub tree
      * @return a hierarchical configuration that contains this sub tree
@@ -466,13 +598,7 @@ public class HierarchicalConfiguration extends AbstractConfiguration implements
      */
     public SubnodeConfiguration configurationAt(String key)
     {
-        List nodes = fetchNodeList(key);
-        if (nodes.size() != 1)
-        {
-            throw new IllegalArgumentException(
-                    "Passed in key must select exactly one node: " + key);
-        }
-        return createSubnodeConfiguration((ConfigurationNode) nodes.get(0));
+        return configurationAt(key, false);
     }
 
     /**
@@ -524,7 +650,60 @@ public class HierarchicalConfiguration extends AbstractConfiguration implements
      */
     protected SubnodeConfiguration createSubnodeConfiguration(ConfigurationNode node)
     {
-        return new SubnodeConfiguration(this, node);
+        SubnodeConfiguration result = new SubnodeConfiguration(this, node);
+        registerSubnodeConfiguration(result);
+        return result;
+    }
+
+    /**
+     * Creates a new subnode configuration for the specified node and sets its
+     * construction key. A subnode configuration created this way will be aware
+     * of structural changes of its parent.
+     *
+     * @param node the node, for which a subnode configuration is to be created
+     * @param subnodeKey the key used to construct the configuration
+     * @return the configuration for the given node
+     * @since 1.5
+     */
+    protected SubnodeConfiguration createSubnodeConfiguration(
+            ConfigurationNode node, String subnodeKey)
+    {
+        SubnodeConfiguration result = createSubnodeConfiguration(node);
+        result.setSubnodeKey(subnodeKey);
+        return result;
+    }
+
+    /**
+     * This method is always called when a subnode configuration created from
+     * this configuration has been modified. This implementation transforms the
+     * received event into an event of type <code>EVENT_SUBNODE_CHANGED</code>
+     * and notifies the registered listeners.
+     *
+     * @param event the event describing the change
+     * @since 1.5
+     */
+    protected void subnodeConfigurationChanged(ConfigurationEvent event)
+    {
+        fireEvent(EVENT_SUBNODE_CHANGED, null, event, event.isBeforeUpdate());
+    }
+
+    /**
+     * Registers this instance at the given subnode configuration. This
+     * implementation will register a change listener, so that modifications of
+     * the subnode configuration can be tracked.
+     *
+     * @param config the subnode configuration
+     * @since 1.5
+     */
+    void registerSubnodeConfiguration(SubnodeConfiguration config)
+    {
+        config.addConfigurationListener(new ConfigurationListener()
+        {
+            public void configurationChanged(ConfigurationEvent event)
+            {
+                subnodeConfigurationChanged(event);
+            }
+        });
     }
 
     /**
@@ -552,6 +731,7 @@ public class HierarchicalConfiguration extends AbstractConfiguration implements
     {
         fireEvent(EVENT_SET_PROPERTY, key, value, true);
 
+        // Update the existing nodes for this property
         Iterator itNodes = fetchNodeList(key).iterator();
         Iterator itValues;
         if (!isDelimiterParsingDisabled())
@@ -562,6 +742,7 @@ public class HierarchicalConfiguration extends AbstractConfiguration implements
         {
             itValues = new SingletonIterator(value);
         }
+
         while (itNodes.hasNext() && itValues.hasNext())
         {
             ((ConfigurationNode) itNodes.next()).setValue(itValues.next());
@@ -709,6 +890,29 @@ public class HierarchicalConfiguration extends AbstractConfiguration implements
         }
     }
 
+    /**
+     * Returns a configuration with the same content as this configuration, but
+     * with all variables replaced by their actual values. This implementation
+     * is specific for hierarchical configurations. It clones the current
+     * configuration and runs a specialized visitor on the clone, which performs
+     * interpolation on the single configuration nodes.
+     *
+     * @return a configuration with all variables interpolated
+     * @since 1.5
+     */
+    public Configuration interpolatedConfiguration()
+    {
+        HierarchicalConfiguration c = (HierarchicalConfiguration) clone();
+        c.getRootNode().visit(new ConfigurationNodeVisitorAdapter()
+        {
+            public void visitAfterChildren(ConfigurationNode node)
+            {
+                node.setValue(interpolate(node.getValue()));
+            }
+        });
+        return c;
+    }
+
     /**
      * Helper method for fetching a list of all nodes that are addressed by the
      * specified key.
@@ -803,7 +1007,7 @@ public class HierarchicalConfiguration extends AbstractConfiguration implements
      * Clears the value of the specified node. If the node becomes undefined by
      * this operation, it is removed from the hierarchy.
      *
-     * @param node the node to be cleard
+     * @param node the node to be cleared
      * @deprecated Use the method <code>{@link #clearNode(ConfigurationNode)}</code>
      * instead
      */
@@ -816,7 +1020,7 @@ public class HierarchicalConfiguration extends AbstractConfiguration implements
      * Clears the value of the specified node. If the node becomes undefined by
      * this operation, it is removed from the hierarchy.
      *
-     * @param node the node to be cleard
+     * @param node the node to be cleared
      */
     protected void clearNode(ConfigurationNode node)
     {
