@@ -18,12 +18,16 @@
 package org.apache.commons.configuration;
 
 import java.awt.Color;
+import java.lang.reflect.Array;
 import java.lang.reflect.Constructor;
 import java.lang.reflect.InvocationTargetException;
+import java.lang.reflect.Method;
 import java.math.BigDecimal;
 import java.math.BigInteger;
+import java.net.InetAddress;
 import java.net.MalformedURLException;
 import java.net.URL;
+import java.net.UnknownHostException;
 import java.text.ParseException;
 import java.text.SimpleDateFormat;
 import java.util.ArrayList;
@@ -39,18 +43,22 @@ import org.apache.commons.collections.iterators.IteratorChain;
 import org.apache.commons.collections.iterators.SingletonIterator;
 import org.apache.commons.lang.BooleanUtils;
 import org.apache.commons.lang.StringUtils;
+import org.apache.commons.lang.SystemUtils;
 
 /**
  * A utility class to convert the configuration properties into any type.
  *
  * @author Emmanuel Bourg
- * @version $Revision: 490321 $, $Date: 2006-12-26 17:19:07 +0100 (Di, 26 Dez 2006) $
+ * @version $Revision: 589380 $, $Date: 2007-10-28 17:37:35 +0100 (So, 28 Okt 2007) $
  * @since 1.1
  */
 public final class PropertyConverter
 {
-    /** Constant for the list delimiter escaping character.*/
-    static final String LIST_ESCAPE = "\\";
+    /** Constant for the list delimiter as char.*/
+    static final char LIST_ESC_CHAR = '\\';
+
+    /** Constant for the list delimiter escaping character as string.*/
+    static final String LIST_ESCAPE = String.valueOf(LIST_ESC_CHAR);
 
     /** Constant for the prefix of hex numbers.*/
     private static final String HEX_PREFIX = "0x";
@@ -58,11 +66,15 @@ public final class PropertyConverter
     /** Constant for the radix of hex numbers.*/
     private static final int HEX_RADIX = 16;
 
-    /** Constant for the argument classes of the Number constructor that takes
-     * a String.
-     */
+    /** Constant for the Java version 1.5.*/
+    private static final float JAVA_VERSION_1_5 = 1.5f;
+
+    /** Constant for the argument classes of the Number constructor that takes a String. */
     private static final Class[] CONSTR_ARGS = {String.class};
 
+    /** The fully qualified name of {@link javax.mail.internet.InternetAddress} */
+    private static final String INTERNET_ADDRESS_CLASSNAME = "javax.mail.internet.InternetAddress";
+
     /**
      * Private constructor prevents instances from being created.
      */
@@ -71,10 +83,101 @@ public final class PropertyConverter
         // to prevent instanciation...
     }
 
+    /**
+     * Converts the specified value to the target class. If the class is a
+     * primitive type (Integer.TYPE, Boolean.TYPE, etc) the value returned
+     * will use the wrapper type (Integer.class, Boolean.class, etc).
+     *
+     * @param cls   the target class of the converted value
+     * @param value the value to convert
+     * @param params optional parameters used for the conversion
+     * @return the converted value
+     * @throws ConversionException if the value is not compatible with the requested type
+     *
+     * @since 1.5
+     */
+    static Object to(Class cls, Object value, Object[] params) throws ConversionException
+    {
+        if (Boolean.class.equals(cls) || Boolean.TYPE.equals(cls))
+        {
+            return toBoolean(value);
+        }
+        else if (Number.class.isAssignableFrom(cls) || cls.isPrimitive())
+        {
+            if (Integer.class.equals(cls) || Integer.TYPE.equals(cls))
+            {
+                return toInteger(value);
+            }
+            else if (Long.class.equals(cls) || Long.TYPE.equals(cls))
+            {
+                return toLong(value);
+            }
+            else if (Byte.class.equals(cls) || Byte.TYPE.equals(cls))
+            {
+                return toByte(value);
+            }
+            else if (Short.class.equals(cls) || Short.TYPE.equals(cls))
+            {
+                return toShort(value);
+            }
+            else if (Float.class.equals(cls) || Float.TYPE.equals(cls))
+            {
+                return toFloat(value);
+            }
+            else if (Double.class.equals(cls) || Double.TYPE.equals(cls))
+            {
+                return toDouble(value);
+            }
+            else if (BigInteger.class.equals(cls))
+            {
+                return toBigInteger(value);
+            }
+            else if (BigDecimal.class.equals(cls))
+            {
+                return toBigDecimal(value);
+            }
+        }
+        else if (Date.class.equals(cls))
+        {
+            return toDate(value, (String) params[0]);
+        }
+        else if (Calendar.class.equals(cls))
+        {
+            return toCalendar(value, (String) params[0]);
+        }
+        else if (URL.class.equals(cls))
+        {
+            return toURL(value);
+        }
+        else if (Locale.class.equals(cls))
+        {
+            return toLocale(value);
+        }
+        else if (isEnum(cls))
+        {
+            return toEnum(value, cls);
+        }
+        else if (Color.class.equals(cls))
+        {
+            return toColor(value);
+        }
+        else if (cls.getName().equals(INTERNET_ADDRESS_CLASSNAME))
+        {
+            return toInternetAddress(value);
+        }
+        else if (InetAddress.class.isAssignableFrom(cls))
+        {
+            return toInetAddress(value);
+        }
+
+        throw new ConversionException("The value '" + value + "' (" + value.getClass() + ")"
+                + " can't be converted to a " + cls.getName() + " object");
+    }
+
     /**
      * Convert the specified object into a Boolean. Internally the
      * <code>org.apache.commons.lang.BooleanUtils</code> class from the
-     * <a href="http://jakarta.apache.org/commons/lang/">Commons Lang</a>
+     * <a href="http://commons.apache.org/lang/">Commons Lang</a>
      * project is used to perform this conversion. This class accepts some more
      * tokens for the boolean value of <b>true</b>, e.g. <code>yes</code> and
      * <code>on</code>. Please refer to the documentation of this class for more
@@ -277,8 +380,7 @@ public final class PropertyConverter
      * @return the converted number
      * @throws ConversionException if the object cannot be converted
      */
-    static Number toNumber(Object value, Class targetClass)
-            throws ConversionException
+    static Number toNumber(Object value, Class targetClass) throws ConversionException
     {
         if (value instanceof Number)
         {
@@ -291,8 +393,7 @@ public final class PropertyConverter
             {
                 try
                 {
-                    return new BigInteger(str.substring(HEX_PREFIX.length()),
-                            HEX_RADIX);
+                    return new BigInteger(str.substring(HEX_PREFIX.length()), HEX_RADIX);
                 }
                 catch (NumberFormatException nex)
                 {
@@ -411,40 +512,54 @@ public final class PropertyConverter
 
         StringBuffer token = new StringBuffer();
         int begin = 0;
-        int end = 0;
-        while (begin <= s.length())
+        boolean inEscape = false;
+
+        while (begin < s.length())
         {
-            // find the next delimiter
-            int index = s.indexOf(delimiter, end);
-
-            // move the end index at the end of the string if the delimiter is not found
-            end = (index != -1) ? index : s.length();
-
-            // extract the chunk
-            String chunk = s.substring(begin , end);
-
-            if (chunk.endsWith(LIST_ESCAPE) && end != s.length())
+            char c = s.charAt(begin);
+            if (inEscape)
             {
-                token.append(chunk.substring(0, chunk.length() - 1));
-                token.append(delimiter);
+                // last character was the escape marker
+                // can current character be escaped?
+                if (c != delimiter && c != LIST_ESC_CHAR)
+                {
+                    // no, also add escape character
+                    token.append(LIST_ESC_CHAR);
+                }
+                token.append(c);
+                inEscape = false;
             }
+
             else
             {
-                // append the chunk to the token
-                token.append(chunk);
-
-                // add the token to the list
-                list.add(token.toString().trim());
-
-                // reset the token
-                token = new StringBuffer();
+                if (c == delimiter)
+                {
+                    // found a list delimiter -> add token and reset buffer
+                    list.add(token.toString().trim());
+                    token = new StringBuffer();
+                }
+                else if (c == LIST_ESC_CHAR)
+                {
+                    // eventually escape next character
+                    inEscape = true;
+                }
+                else
+                {
+                    token.append(c);
+                }
             }
 
-            // move to the next chunk
-            end = end + 1;
-            begin = end;
+            begin++;
         }
 
+        // Trailing delimiter?
+        if (inEscape)
+        {
+            token.append(LIST_ESC_CHAR);
+        }
+        // Add last token
+        list.add(token.toString().trim());
+
         return list;
     }
 
@@ -453,7 +568,7 @@ public final class PropertyConverter
      * method ensures that list delimiter characters that are part of a
      * property's value are correctly escaped when a configuration is saved to a
      * file. Otherwise when loaded again the property will be treated as a list
-     * property.
+     * property. A single backslash will also be escaped.
      *
      * @param s the string with the value
      * @param delimiter the list delimiter to use
@@ -461,8 +576,8 @@ public final class PropertyConverter
      */
     public static String escapeDelimiters(String s, char delimiter)
     {
-        return StringUtils.replace(s, String.valueOf(delimiter), LIST_ESCAPE
-                + delimiter);
+        String s1 = StringUtils.replace(s, LIST_ESCAPE, LIST_ESCAPE + LIST_ESCAPE);
+        return StringUtils.replace(s1, String.valueOf(delimiter), LIST_ESCAPE + delimiter);
     }
 
     /**
@@ -536,6 +651,141 @@ public final class PropertyConverter
         }
     }
 
+    /**
+     * Convert the specified value into an internet address.
+     *
+     * @param value the value to convert
+     * @return the converted value
+     * @throws ConversionException thrown if the value cannot be converted to a InetAddress
+     *
+     * @since 1.5
+     */
+    static InetAddress toInetAddress(Object value) throws ConversionException
+    {
+        if (value instanceof InetAddress)
+        {
+            return (InetAddress) value;
+        }
+        else if (value instanceof String)
+        {
+            try
+            {
+                return InetAddress.getByName((String) value);
+            }
+            catch (UnknownHostException e)
+            {
+                throw new ConversionException("The value " + value + " can't be converted to a InetAddress", e);
+            }
+        }
+        else
+        {
+            throw new ConversionException("The value " + value + " can't be converted to a InetAddress");
+        }
+    }
+
+    /**
+     * Convert the specified value into an email address.
+     *
+     * @param value the value to convert
+     * @return the converted value
+     * @throws ConversionException thrown if the value cannot be converted to an email address
+     *
+     * @since 1.5
+     */
+    static Object toInternetAddress(Object value) throws ConversionException
+    {
+        if (value.getClass().getName().equals(INTERNET_ADDRESS_CLASSNAME))
+        {
+            return value;
+        }
+        else if (value instanceof String)
+        {
+            try
+            {
+                Constructor ctor = Class.forName(INTERNET_ADDRESS_CLASSNAME).getConstructor(new Class[] {String.class});
+                return ctor.newInstance(new Object[] {value});
+            }
+            catch (Exception e)
+            {
+                throw new ConversionException("The value " + value + " can't be converted to a InternetAddress", e);
+            }
+        }
+        else
+        {
+            throw new ConversionException("The value " + value + " can't be converted to a InternetAddress");
+        }
+    }
+
+    /**
+     * Calls Class.isEnum() on Java 5, returns false on older JRE.
+     */
+    static boolean isEnum(Class cls)
+    {
+        if (!SystemUtils.isJavaVersionAtLeast(JAVA_VERSION_1_5))
+        {
+            return false;
+        }
+
+        try
+        {
+            Method isEnumMethod = Class.class.getMethod("isEnum", new Class[] {});
+            return ((Boolean) isEnumMethod.invoke(cls, new Object[] {})).booleanValue();
+        }
+        catch (Exception e)
+        {
+            // impossible
+            throw new RuntimeException(e.getMessage());
+        }
+    }
+
+    /**
+     * Convert the specified value into a Java 5 enum.
+     *
+     * @param value the value to convert
+     * @param cls   the type of the enumeration
+     * @return the converted value
+     * @throws ConversionException thrown if the value cannot be converted to an enumeration
+     *
+     * @since 1.5
+     */
+    static Object toEnum(Object value, Class cls) throws ConversionException
+    {
+        if (value.getClass().equals(cls))
+        {
+            return value;
+        }
+        else if (value instanceof String)
+        {
+            try
+            {
+                Method valueOfMethod = cls.getMethod("valueOf", new Class[] {String.class});
+                return valueOfMethod.invoke(null, new Object[] {value});
+            }
+            catch (Exception e)
+            {
+                throw new ConversionException("The value " + value + " can't be converted to a " + cls.getName());
+            }
+        }
+        else if (value instanceof Number)
+        {
+            try
+            {
+                Method valuesMethod = cls.getMethod("values", new Class[] {});
+                Object valuesArray = valuesMethod.invoke(null, new Object[] {});
+
+                return Array.get(valuesArray, ((Number) value).intValue());
+            }
+            catch (Exception e)
+            {
+                throw new ConversionException("The value " + value + " can't be converted to a " + cls.getName());
+            }
+        }
+        else
+        {
+            throw new ConversionException("The value " + value + " can't be converted to a " + cls.getName());
+        }
+    }
+
     /**
      * Convert the specified object into a Date.
      *
