@@ -27,7 +27,6 @@ import javax.sql.DataSource;
 
 import junit.framework.TestCase;
 
-import org.apache.commons.configuration.event.ConfigurationErrorEvent;
 import org.apache.commons.configuration.event.ConfigurationErrorListener;
 import org.apache.commons.configuration.test.HsqlDB;
 import org.apache.commons.dbcp.BasicDataSource;
@@ -42,7 +41,7 @@ import org.dbunit.operation.DatabaseOperation;
  * Test in Eclipse it sometimes takes a couple tries. Otherwise you may get
  * database is already in use by another process errors.
  *
- * @version $Revision: 514234 $, $Date: 2007-03-03 21:18:14 +0100 (Sa, 03 Mrz 2007) $
+ * @version $Revision: 558330 $, $Date: 2007-07-21 17:26:19 +0200 (Sa, 21 Jul 2007) $
  */
 public class TestDatabaseConfiguration extends TestCase
 {
@@ -74,7 +73,7 @@ public class TestDatabaseConfiguration extends TestCase
     private DataSource datasource;
 
     /** An error listener for testing whether internal errors occurred.*/
-    private TestErrorListener listener;
+    private ConfigurationErrorListenerImpl listener;
 
     protected void setUp() throws Exception
     {
@@ -121,7 +120,7 @@ public class TestDatabaseConfiguration extends TestCase
         // if an error listener is defined, we check whether an error occurred
         if(listener != null)
         {
-            assertEquals("An internal error occurred", 0, listener.errorCount);
+            assertEquals("An internal error occurred", 0, listener.getErrorCount());
         }
         super.tearDown();
     }
@@ -133,8 +132,7 @@ public class TestDatabaseConfiguration extends TestCase
      */
     private PotentialErrorDatabaseConfiguration setUpConfig()
     {
-        return new PotentialErrorDatabaseConfiguration(datasource, TABLE,
-                COL_KEY, COL_VALUE);
+        return new PotentialErrorDatabaseConfiguration(datasource, TABLE, COL_KEY, COL_VALUE);
     }
 
     /**
@@ -145,8 +143,7 @@ public class TestDatabaseConfiguration extends TestCase
      */
     private DatabaseConfiguration setUpMultiConfig()
     {
-        return new DatabaseConfiguration(datasource, TABLE_MULTI, COL_NAME,
-                COL_KEY, COL_VALUE, CONFIG_NAME);
+        return new DatabaseConfiguration(datasource, TABLE_MULTI, COL_NAME, COL_KEY, COL_VALUE, CONFIG_NAME);
     }
 
     /**
@@ -157,9 +154,8 @@ public class TestDatabaseConfiguration extends TestCase
     private void setUpErrorListener(PotentialErrorDatabaseConfiguration config)
     {
         // remove log listener to avoid exception longs
-        config.removeErrorListener((ConfigurationErrorListener) config
-                .getErrorListeners().iterator().next());
-        listener = new TestErrorListener();
+        config.removeErrorListener((ConfigurationErrorListener) config.getErrorListeners().iterator().next());
+        listener = new ConfigurationErrorListenerImpl();
         config.addErrorListener(listener);
         config.failOnConnect = true;
     }
@@ -187,18 +183,12 @@ public class TestDatabaseConfiguration extends TestCase
      */
     private void checkErrorListener(int type, String key, Object value)
     {
-        assertEquals("Wrong number of errors", 1, listener.errorCount);
-        assertEquals("Wrong event type", type, listener.event.getType());
-        assertTrue("Wrong event source",
-                listener.event.getSource() instanceof DatabaseConfiguration);
+        listener.verify(type, key, value);
+        assertTrue(
+                "Wrong event source",
+                listener.getLastEvent().getSource() instanceof DatabaseConfiguration);
         assertTrue("Wrong exception",
-                listener.event.getCause() instanceof SQLException);
-        assertTrue("Wrong property key", (key == null) ? listener.event
-                .getPropertyName() == null : key.equals(listener.event
-                .getPropertyName()));
-        assertTrue("Wrong property value", (value == null) ? listener.event
-                .getPropertyValue() == null : value.equals(listener.event
-                .getPropertyValue()));
+                listener.getLastEvent().getCause() instanceof SQLException);
         listener = null; // mark as checked
     }
 
@@ -357,10 +347,8 @@ public class TestDatabaseConfiguration extends TestCase
      */
     public void testLogErrorListener()
     {
-        DatabaseConfiguration config = new DatabaseConfiguration(datasource,
-                TABLE, COL_KEY, COL_VALUE);
-        assertEquals("No error listener registered", 1, config
-                .getErrorListeners().size());
+        DatabaseConfiguration config = new DatabaseConfiguration(datasource, TABLE, COL_KEY, COL_VALUE);
+        assertEquals("No error listener registered", 1, config.getErrorListeners().size());
     }
 
     /**
@@ -369,8 +357,7 @@ public class TestDatabaseConfiguration extends TestCase
     public void testGetPropertyError()
     {
         setUpErrorConfig().getProperty("key1");
-        checkErrorListener(AbstractConfiguration.EVENT_READ_PROPERTY, "key1",
-                null);
+        checkErrorListener(AbstractConfiguration.EVENT_READ_PROPERTY, "key1", null);
     }
 
     /**
@@ -379,8 +366,7 @@ public class TestDatabaseConfiguration extends TestCase
     public void testAddPropertyError()
     {
         setUpErrorConfig().addProperty("key1", "value");
-        checkErrorListener(AbstractConfiguration.EVENT_ADD_PROPERTY, "key1",
-                "value");
+        checkErrorListener(AbstractConfiguration.EVENT_ADD_PROPERTY, "key1", "value");
     }
 
     /**
@@ -388,10 +374,8 @@ public class TestDatabaseConfiguration extends TestCase
      */
     public void testIsEmptyError()
     {
-        assertTrue("Wrong return value for failure", setUpErrorConfig()
-                .isEmpty());
-        checkErrorListener(AbstractConfiguration.EVENT_READ_PROPERTY, null,
-                null);
+        assertTrue("Wrong return value for failure", setUpErrorConfig().isEmpty());
+        checkErrorListener(AbstractConfiguration.EVENT_READ_PROPERTY, null, null);
     }
 
     /**
@@ -399,10 +383,8 @@ public class TestDatabaseConfiguration extends TestCase
      */
     public void testContainsKeyError()
     {
-        assertFalse("Wrong return value for failure", setUpErrorConfig()
-                .containsKey("key1"));
-        checkErrorListener(AbstractConfiguration.EVENT_READ_PROPERTY, "key1",
-                null);
+        assertFalse("Wrong return value for failure", setUpErrorConfig().containsKey("key1"));
+        checkErrorListener(AbstractConfiguration.EVENT_READ_PROPERTY, "key1", null);
     }
 
     /**
@@ -411,8 +393,7 @@ public class TestDatabaseConfiguration extends TestCase
     public void testClearPropertyError()
     {
         setUpErrorConfig().clearProperty("key1");
-        checkErrorListener(AbstractConfiguration.EVENT_CLEAR_PROPERTY, "key1",
-                null);
+        checkErrorListener(AbstractConfiguration.EVENT_CLEAR_PROPERTY, "key1", null);
     }
 
     /**
@@ -430,8 +411,7 @@ public class TestDatabaseConfiguration extends TestCase
     public void testGetKeysError()
     {
         Iterator it = setUpErrorConfig().getKeys();
-        checkErrorListener(AbstractConfiguration.EVENT_READ_PROPERTY, null,
-                null);
+        checkErrorListener(AbstractConfiguration.EVENT_READ_PROPERTY, null, null);
         assertFalse("Iteration is not empty", it.hasNext());
     }
 
@@ -458,8 +438,7 @@ public class TestDatabaseConfiguration extends TestCase
         DatabaseConfiguration config = setUpConfig();
         config.setListDelimiter(';');
         config.setDelimiterParsingDisabled(true);
-        assertEquals("Wrong value of property", "a;b;c", config
-                .getString("keyMulti"));
+        assertEquals("Wrong value of property", "a;b;c", config.getString("keyMulti"));
     }
 
     /**
@@ -481,8 +460,7 @@ public class TestDatabaseConfiguration extends TestCase
      * configured to throw an exception when obtaining a connection. This way
      * database exceptions can be simulated.
      */
-    static class PotentialErrorDatabaseConfiguration extends
-            DatabaseConfiguration
+    static class PotentialErrorDatabaseConfiguration extends DatabaseConfiguration
     {
         /** A flag whether a getConnection() call should fail. */
         boolean failOnConnect;
@@ -502,23 +480,4 @@ public class TestDatabaseConfiguration extends TestCase
             return super.getConnection();
         }
     }
-
-    /**
-     * A test error listener implementation that is used for finding out whether
-     * error events are correctly triggered.
-     */
-    static class TestErrorListener implements ConfigurationErrorListener
-    {
-        /** Stores the number of calls. */
-        int errorCount;
-
-        /** Stores the last error event. */
-        ConfigurationErrorEvent event;
-
-        public void configurationError(ConfigurationErrorEvent event)
-        {
-            errorCount++;
-            this.event = event;
-        }
-    }
 }
