@@ -26,8 +26,6 @@ import java.io.FileWriter;
 import java.io.IOException;
 import java.io.PrintWriter;
 
-import org.apache.commons.configuration.event.ConfigurationErrorEvent;
-import org.apache.commons.configuration.event.ConfigurationErrorListener;
 import org.apache.commons.configuration.reloading.FileAlwaysReloadingStrategy;
 import org.apache.commons.configuration.reloading.FileChangedReloadingStrategy;
 
@@ -35,22 +33,32 @@ import junit.framework.TestCase;
 
 /**
  * @author Emmanuel Bourg
- * @version $Revision: 497574 $, $Date: 2007-01-18 22:02:55 +0100 (Do, 18 Jan 2007) $
+ * @version $Revision: 561230 $, $Date: 2007-07-31 06:17:09 +0200 (Di, 31 Jul 2007) $
  */
 public class TestFileConfiguration extends TestCase
 {
+    /** Constant for the output directory.*/
     private static final File TARGET_DIR = new File("target");
 
+    /** Constant for the directory with the test configuration files.*/
+    private static final File TEST_DIR = new File("conf");
+
+    /** Constant for the name of a test file.*/
+    private static final String TEST_FILENAME = "test.properties";
+
+    /** Constant for a test file.*/
+    private static final File TEST_FILE = new File(TEST_DIR, TEST_FILENAME);
+
+    /** Constant for the name of a resource to be resolved.*/
     private static final String RESOURCE_NAME = "config/deep/deeptest.properties";
 
     public void testSetURL() throws Exception
     {
         // http URL
         FileConfiguration config = new PropertiesConfiguration();
-        config.setURL(new URL("http://jakarta.apache.org/commons/configuration/index.html"));
+        config.setURL(new URL("http://commons.apache.org/configuration/index.html"));
 
-        assertEquals("base path", "http://jakarta.apache.org/commons/configuration/", config
-                .getBasePath());
+        assertEquals("base path", "http://commons.apache.org/configuration/", config.getBasePath());
         assertEquals("file name", "index.html", config.getFileName());
 
         // file URL
@@ -62,13 +70,10 @@ public class TestFileConfiguration extends TestCase
     public void testSetURLWithParams() throws Exception
     {
         FileConfiguration config = new PropertiesConfiguration();
-        URL url = new URL(
-                "http://issues.apache.org/bugzilla/show_bug.cgi?id=37886");
+        URL url = new URL("http://issues.apache.org/bugzilla/show_bug.cgi?id=37886");
         config.setURL(url);
-        assertEquals("Base path incorrect",
-                "http://issues.apache.org/bugzilla/", config.getBasePath());
-        assertEquals("File name incorrect", "show_bug.cgi", config
-                .getFileName());
+        assertEquals("Base path incorrect", "http://issues.apache.org/bugzilla/", config.getBasePath());
+        assertEquals("File name incorrect", "show_bug.cgi", config.getFileName());
         assertEquals("URL was not correctly stored", url, config.getURL());
     }
 
@@ -76,23 +81,23 @@ public class TestFileConfiguration extends TestCase
     {
         PropertiesConfiguration config = new PropertiesConfiguration();
 
-        File directory = new File("conf");
-        File file = new File(directory, "test.properties");
+        File directory = TEST_DIR;
+        File file = TEST_FILE;
         config.setFile(file);
         assertEquals(directory.getAbsolutePath(), config.getBasePath());
-        assertEquals("test.properties", config.getFileName());
+        assertEquals(TEST_FILENAME, config.getFileName());
         assertEquals(file.getAbsolutePath(), config.getPath());
 
-        config.setPath("conf" + File.separator + "test.properties");
-        assertEquals("test.properties", config.getFileName());
+        config.setPath("conf" + File.separator + TEST_FILENAME);
+        assertEquals(TEST_FILENAME, config.getFileName());
         assertEquals(directory.getAbsolutePath(), config.getBasePath());
         assertEquals(file.getAbsolutePath(), config.getPath());
         assertEquals(file.toURL(), config.getURL());
 
         config.setBasePath(null);
-        config.setFileName("test.properties");
+        config.setFileName(TEST_FILENAME);
         assertNull(config.getBasePath());
-        assertEquals("test.properties", config.getFileName());
+        assertEquals(TEST_FILENAME, config.getFileName());
     }
 
     public void testCreateFile1() throws Exception
@@ -202,9 +207,10 @@ public class TestFileConfiguration extends TestCase
     public void testSaveInvalidURL() throws Exception
     {
         FileConfiguration config = new PropertiesConfiguration();
+
         try
         {
-            config.save(new URL("http://jakarta.apache.org"));
+            config.save(new URL("http://jakarta.apache.org/test.properties"));
             fail("Should throw a ConfigurationException!");
         }
         catch (ConfigurationException cex)
@@ -214,7 +220,7 @@ public class TestFileConfiguration extends TestCase
 
         try
         {
-            config.save("http://www.apache.org");
+            config.save("http://www.apache.org/test.properties");
             fail("Should throw a ConfigurationException!");
         }
         catch (ConfigurationException cex)
@@ -292,7 +298,7 @@ public class TestFileConfiguration extends TestCase
      */
     public void testReloadingWithAutoSave() throws Exception
     {
-        File configFile = new File(TARGET_DIR, "test.properties");
+        File configFile = new File(TARGET_DIR, TEST_FILENAME);
         PrintWriter out = null;
 
         try
@@ -382,13 +388,43 @@ public class TestFileConfiguration extends TestCase
     {
         FileConfiguration config = new PropertiesConfiguration();
         assertNull(config.getFile());
-        File file = new File("conf/test.properties").getAbsoluteFile();
+        File file = TEST_FILE.getAbsoluteFile();
         config.setFile(file);
         assertEquals(file, config.getFile());
         config.load();
         assertEquals(file, config.getFile());
     }
 
+    /**
+     * Tests whether getFile() returns a valid file after a configuration has
+     * been loaded.
+     */
+    public void testGetFileAfterLoad() throws ConfigurationException,
+            IOException
+    {
+        FileConfiguration config = new PropertiesConfiguration();
+        config.load(TEST_FILE.getAbsolutePath());
+        assertNotNull("No source URL set", config.getURL());
+        assertEquals("Wrong source file", TEST_FILE.getCanonicalFile(), config
+                .getFile().getCanonicalFile());
+    }
+
+    /**
+     * Tests whether calling load() multiple times changes the source. This
+     * should not be the case.
+     */
+    public void testLoadMultiple() throws ConfigurationException
+    {
+        FileConfiguration config = new PropertiesConfiguration();
+        config.load(TEST_FILE.getAbsolutePath());
+        URL srcUrl = config.getURL();
+        File srcFile = config.getFile();
+        File file2 = new File(TEST_DIR, "testEqual.properties");
+        config.load(file2.getAbsolutePath());
+        assertEquals("Source URL was changed", srcUrl, config.getURL());
+        assertEquals("Source file was changed", srcFile, config.getFile());
+    }
+
     /**
      * Tests to invoke save() without explicitely setting a file name. This
      * will cause an exception.
@@ -396,7 +432,7 @@ public class TestFileConfiguration extends TestCase
     public void testSaveWithoutFileName() throws Exception
     {
         FileConfiguration config = new PropertiesConfiguration();
-        File file = new File("conf/test.properties");
+        File file = TEST_FILE;
         config.load(file);
         try
         {
@@ -409,7 +445,7 @@ public class TestFileConfiguration extends TestCase
         }
 
         config = new PropertiesConfiguration();
-        config.load("conf/test.properties");
+        config.load(TEST_FILE);
         try
         {
             config.save();
@@ -496,12 +532,12 @@ public class TestFileConfiguration extends TestCase
     }
 
     /**
-     * Tests the loading of configuration file in a Combined configuration 
+     * Tests the loading of configuration file in a Combined configuration
      * when the configuration source is in the classpath.
      */
     public void testLoadFromClassPath() throws ConfigurationException
     {
-        DefaultConfigurationBuilder cf = 
+        DefaultConfigurationBuilder cf =
             new DefaultConfigurationBuilder("conf/config/deep/testFileFromClasspath.xml");
         CombinedConfiguration config = cf.getConfiguration(true);
         Configuration config1 = config.getConfiguration("propConf");
@@ -540,20 +576,7 @@ public class TestFileConfiguration extends TestCase
      */
     public void testReloadError() throws ConfigurationException
     {
-        class TestConfigurationErrorListener implements
-                ConfigurationErrorListener
-        {
-            ConfigurationErrorEvent event;
-
-            int errorCount;
-
-            public void configurationError(ConfigurationErrorEvent event)
-            {
-                this.event = event;
-                errorCount++;
-            }
-        };
-        TestConfigurationErrorListener l = new TestConfigurationErrorListener();
+        ConfigurationErrorListenerImpl l = new ConfigurationErrorListenerImpl();
         PropertiesConfiguration config = new PropertiesConfiguration(
                 RESOURCE_NAME);
         config.clearErrorListeners();
@@ -562,12 +585,8 @@ public class TestFileConfiguration extends TestCase
         config.getString("test");
         config.setFileName("Not existing file");
         config.getString("test");
-        assertEquals("Wrong number of error events", 1, l.errorCount);
-        assertEquals("Wrong error type",
-                AbstractFileConfiguration.EVENT_RELOAD, l.event.getType());
-        assertNull("Wrong property name", l.event.getPropertyName());
-        assertNull("Wrong property value", l.event.getPropertyValue());
-        assertNotNull("Exception is not set", l.event.getCause());
+        l.verify(AbstractFileConfiguration.EVENT_RELOAD, null, null);
+        assertNotNull("Exception is not set", l.getLastEvent().getCause());
     }
 
     /**
