@@ -18,12 +18,18 @@
 package org.apache.commons.configuration;
 
 import java.io.File;
+import java.io.FileOutputStream;
 import java.io.FileWriter;
 import java.io.IOException;
+import java.io.OutputStream;
 import java.io.PrintWriter;
 import java.io.Reader;
 import java.io.StringReader;
 import java.io.StringWriter;
+import java.net.HttpURLConnection;
+import java.net.URL;
+import java.net.URLConnection;
+import java.net.URLStreamHandler;
 import java.util.ArrayList;
 import java.util.Iterator;
 import java.util.List;
@@ -35,7 +41,7 @@ import junit.framework.TestCase;
 /**
  * Test for loading and saving properties files.
  *
- * @version $Id: TestPropertiesConfiguration.java 439648 2006-09-02 20:42:10Z oheger $
+ * @version $Id: TestPropertiesConfiguration.java 567771 2007-08-20 17:57:08Z oheger $
  */
 public class TestPropertiesConfiguration extends TestCase
 {
@@ -51,6 +57,13 @@ public class TestPropertiesConfiguration extends TestCase
     protected void setUp() throws Exception
     {
         conf = new PropertiesConfiguration(testProperties);
+
+        // remove the test save file if it exists
+        if (testSavePropertiesFile.exists())
+        {
+            assertTrue("Test output file could not be deleted",
+                    testSavePropertiesFile.delete());
+        }
     }
 
     public void testLoad() throws Exception
@@ -126,12 +139,6 @@ public class TestPropertiesConfiguration extends TestCase
 
     public void testSave() throws Exception
     {
-        // remove the file previously saved if necessary
-        if (testSavePropertiesFile.exists())
-        {
-            assertTrue(testSavePropertiesFile.delete());
-        }
-
         // add an array of strings to the configuration
         conf.addProperty("string", "value1");
         List list = new ArrayList();
@@ -149,25 +156,25 @@ public class TestPropertiesConfiguration extends TestCase
 
         // read the configuration and compare the properties
         PropertiesConfiguration checkConfig = new PropertiesConfiguration(filename);
-        for (Iterator i = conf.getKeys(); i.hasNext();)
-        {
-            String key = (String) i.next();
-            assertTrue("The saved configuration doesn't contain the key '" + key + "'", checkConfig.containsKey(key));
-            assertEquals("Value of the '" + key + "' property", conf.getProperty(key), checkConfig.getProperty(key));
-        }
+        ConfigurationAssert.assertEquals(conf, checkConfig);
 
         // Save it again, verifing a save with a filename works.
         checkConfig.save();
     }
 
+    public void testSaveToCustomURL() throws Exception
+    {
+        // save the configuration to a custom URL
+        URL url = new URL("foo", "", 0, "./target/testsave-custom-url.properties", new FileURLStreamHandler());
+        conf.save(url);
+
+        // reload the configuration
+        Configuration config2 = new PropertiesConfiguration(url);
+        assertEquals("true", config2.getString("configuration.loaded"));
+    }
+
     public void testInMemoryCreatedSave() throws Exception
     {
-        // remove the file previously saved if necessary
-        if (testSavePropertiesFile.exists())
-        {
-            assertTrue(testSavePropertiesFile.delete());
-        }
-
         PropertiesConfiguration pc = new PropertiesConfiguration();
         // add an array of strings to the configuration
         pc.addProperty("string", "value1");
@@ -186,19 +193,30 @@ public class TestPropertiesConfiguration extends TestCase
 
         // read the configuration and compare the properties
         PropertiesConfiguration checkConfig = new PropertiesConfiguration(filename);
-        for (Iterator i = pc.getKeys(); i.hasNext();)
-        {
-            String key = (String) i.next();
-            assertTrue("The saved configuration doesn't contain the key '" + key + "'",
-                    checkConfig.containsKey(key));
-            assertEquals("Value of the '" + key + "' property",
-                    pc.getProperty(key), checkConfig.getProperty(key));
-        }
+        ConfigurationAssert.assertEquals(pc, checkConfig);
 
         // Save it again, verifing a save with a filename works.
         checkConfig.save();
     }
 
+    /**
+     * Tests saving a configuration when delimiter parsing is disabled.
+     */
+    public void testSaveWithDelimiterParsingDisabled() throws ConfigurationException
+    {
+        conf.clear();
+        conf.setDelimiterParsingDisabled(true);
+        conf.addProperty("test.list", "a,b,c");
+        conf.addProperty("test.dirs", "C:\\Temp\\,D:\\Data\\");
+        conf.save(testSavePropertiesFile);
+
+        PropertiesConfiguration checkConfig = new PropertiesConfiguration();
+        checkConfig.setDelimiterParsingDisabled(true);
+        checkConfig.setFile(testSavePropertiesFile);
+        checkConfig.load();
+        ConfigurationAssert.assertEquals(conf, checkConfig);
+    }
+
     public void testSaveMissingFilename()
     {
         PropertiesConfiguration pc = new PropertiesConfiguration();
@@ -219,12 +237,6 @@ public class TestPropertiesConfiguration extends TestCase
      */
     public void testSaveWithBasePath() throws Exception
     {
-        // remove the file previously saved if necessary
-        if (testSavePropertiesFile.exists())
-        {
-            assertTrue(testSavePropertiesFile.delete());
-        }
-
         conf.setProperty("test", "true");
         conf.setBasePath(testSavePropertiesFile.getParentFile().toURL().toString());
         conf.setFileName(testSavePropertiesFile.getName());
@@ -232,6 +244,23 @@ public class TestPropertiesConfiguration extends TestCase
         assertTrue(testSavePropertiesFile.exists());
     }
 
+    /**
+     * Tests whether the escape character for list delimiters can be itself
+     * escaped and survives a save operation.
+     */
+    public void testSaveEscapedEscapingCharacter()
+            throws ConfigurationException
+    {
+        conf.addProperty("test.dirs", "C:\\Temp\\\\,D:\\Data\\\\,E:\\Test\\");
+        List dirs = conf.getList("test.dirs");
+        assertEquals("Wrong number of list elements", 3, dirs.size());
+        conf.save(testSavePropertiesFile);
+
+        PropertiesConfiguration checkConfig = new PropertiesConfiguration(
+                testSavePropertiesFile);
+        ConfigurationAssert.assertEquals(conf, checkConfig);
+    }
+
     public void testLoadViaProperty() throws Exception
     {
         PropertiesConfiguration pc = new PropertiesConfiguration();
@@ -337,8 +366,7 @@ public class TestPropertiesConfiguration extends TestCase
         setUpSavedProperties();
         conf.clearProperty("c");
         PropertiesConfiguration checkConfig = checkSavedConfig();
-        assertFalse("The saved configuration contain the key '" + "c" + "'",
-                checkConfig.containsKey("c"));
+        assertFalse("The saved configuration contain the key '" + "c" + "'", checkConfig.containsKey("c"));
     }
 
     /**
@@ -347,8 +375,7 @@ public class TestPropertiesConfiguration extends TestCase
      *
      * @throws IOException if an I/O error occurs
      */
-    private void setUpSavedProperties() throws IOException,
-            ConfigurationException
+    private void setUpSavedProperties() throws IOException, ConfigurationException
     {
         PrintWriter out = null;
 
@@ -388,15 +415,12 @@ public class TestPropertiesConfiguration extends TestCase
     private PropertiesConfiguration checkSavedConfig()
             throws ConfigurationException
     {
-        PropertiesConfiguration checkConfig = new PropertiesConfiguration(
-                testSavePropertiesFile);
+        PropertiesConfiguration checkConfig = new PropertiesConfiguration(testSavePropertiesFile);
         for (Iterator i = conf.getKeys(); i.hasNext();)
         {
             String key = (String) i.next();
-            assertTrue("The saved configuration doesn't contain the key '"
-                    + key + "'", checkConfig.containsKey(key));
-            assertEquals("Value of the '" + key + "' property", conf
-                    .getProperty(key), checkConfig.getProperty(key));
+            assertTrue("The saved configuration doesn't contain the key '" + key + "'", checkConfig.containsKey(key));
+            assertEquals("Value of the '" + key + "' property", conf.getProperty(key), checkConfig.getProperty(key));
         }
         return checkConfig;
     }
@@ -563,8 +587,7 @@ public class TestPropertiesConfiguration extends TestCase
         String content = out.toString();
         assertTrue("Header could not be found", content.indexOf("# My header"
                 + EOL + EOL) == 0);
-        assertTrue("Property could not be found", content
-                .indexOf("prop = value" + EOL) > 0);
+        assertTrue("Property could not be found", content.indexOf("prop = value" + EOL) > 0);
     }
 
     /**
@@ -607,8 +630,7 @@ public class TestPropertiesConfiguration extends TestCase
         conf.setLayout(layout);
         conf.propertyLoaded("layoutLoadedProperty", "yes");
         assertEquals("Layout's load() was called", 0, layout.loadCalls);
-        assertEquals("Property not added", "yes", conf
-                .getString("layoutLoadedProperty"));
+        assertEquals("Property not added", "yes", conf.getString("layoutLoadedProperty"));
     }
 
     /**
@@ -618,29 +640,23 @@ public class TestPropertiesConfiguration extends TestCase
     {
         DummyLayout layout = new DummyLayout(conf);
         conf.setLayout(layout);
-        conf.propertyLoaded(PropertiesConfiguration.getInclude(),
-                "testClasspath.properties,testEqual.properties");
-        assertEquals("Layout's load() was not correctly called", 2,
-                layout.loadCalls);
-        assertFalse("Property was added", conf
-                .containsKey(PropertiesConfiguration.getInclude()));
+        conf.propertyLoaded(PropertiesConfiguration.getInclude(), "testClasspath.properties,testEqual.properties");
+        assertEquals("Layout's load() was not correctly called", 2, layout.loadCalls);
+        assertFalse("Property was added", conf.containsKey(PropertiesConfiguration.getInclude()));
     }
 
     /**
      * Tests propertyLoaded() for an include property, when includes are
      * disabled.
      */
-    public void testPropertyLoadedIncludeNotAllowed()
-            throws ConfigurationException
+    public void testPropertyLoadedIncludeNotAllowed() throws ConfigurationException
     {
         DummyLayout layout = new DummyLayout(conf);
         conf.setLayout(layout);
         conf.setIncludesAllowed(false);
-        conf.propertyLoaded(PropertiesConfiguration.getInclude(),
-                "testClassPath.properties,testEqual.properties");
+        conf.propertyLoaded(PropertiesConfiguration.getInclude(), "testClassPath.properties,testEqual.properties");
         assertEquals("Layout's load() was called", 0, layout.loadCalls);
-        assertFalse("Property was added", conf
-                .containsKey(PropertiesConfiguration.getInclude()));
+        assertFalse("Property was added", conf.containsKey(PropertiesConfiguration.getInclude()));
     }
 
     /**
@@ -648,16 +664,11 @@ public class TestPropertiesConfiguration extends TestCase
      */
     public void testIsCommentLine()
     {
-        assertTrue("Comment not detected", PropertiesConfiguration
-                .isCommentLine("# a comment"));
-        assertTrue("Alternative comment not detected", PropertiesConfiguration
-                .isCommentLine("! a comment"));
-        assertTrue("Comment with no space not detected",
-                PropertiesConfiguration.isCommentLine("#a comment"));
-        assertTrue("Comment with leading space not detected",
-                PropertiesConfiguration.isCommentLine("    ! a comment"));
-        assertFalse("Wrong comment", PropertiesConfiguration
-                .isCommentLine("   a#comment"));
+        assertTrue("Comment not detected", PropertiesConfiguration.isCommentLine("# a comment"));
+        assertTrue("Alternative comment not detected", PropertiesConfiguration.isCommentLine("! a comment"));
+        assertTrue("Comment with no space not detected", PropertiesConfiguration.isCommentLine("#a comment"));
+        assertTrue("Comment with leading space not detected", PropertiesConfiguration.isCommentLine("    ! a comment"));
+        assertFalse("Wrong comment", PropertiesConfiguration.isCommentLine("   a#comment"));
     }
 
     /**
@@ -667,22 +678,16 @@ public class TestPropertiesConfiguration extends TestCase
     public void testClone() throws ConfigurationException
     {
         PropertiesConfiguration copy = (PropertiesConfiguration) conf.clone();
-        assertNotSame("Copy has same layout object", conf.getLayout(), copy
-                .getLayout());
-        assertEquals("Wrong number of event listeners for original", 1, conf
-                .getConfigurationListeners().size());
-        assertEquals("Wrong number of event listeners for clone", 1, copy
-                .getConfigurationListeners().size());
-        assertSame("Wrong event listener for original", conf.getLayout(), conf
-                .getConfigurationListeners().iterator().next());
-        assertSame("Wrong event listener for clone", copy.getLayout(), copy
-                .getConfigurationListeners().iterator().next());
+        assertNotSame("Copy has same layout object", conf.getLayout(), copy.getLayout());
+        assertEquals("Wrong number of event listeners for original", 1, conf.getConfigurationListeners().size());
+        assertEquals("Wrong number of event listeners for clone", 1, copy.getConfigurationListeners().size());
+        assertSame("Wrong event listener for original", conf.getLayout(), conf.getConfigurationListeners().iterator().next());
+        assertSame("Wrong event listener for clone", copy.getLayout(), copy.getConfigurationListeners().iterator().next());
         StringWriter outConf = new StringWriter();
         conf.save(outConf);
         StringWriter outCopy = new StringWriter();
         copy.save(outCopy);
-        assertEquals("Output from copy is different", outConf.toString(),
-                outCopy.toString());
+        assertEquals("Output from copy is different", outConf.toString(), outCopy.toString());
     }
 
     /**
@@ -692,8 +697,123 @@ public class TestPropertiesConfiguration extends TestCase
     {
         conf = new PropertiesConfiguration();
         PropertiesConfiguration copy = (PropertiesConfiguration) conf.clone();
-        assertNotSame("Layout objects are the same", conf.getLayout(), copy
-                .getLayout());
+        assertNotSame("Layout objects are the same", conf.getLayout(), copy.getLayout());
+    }
+
+    /**
+     * Tests saving a file-based configuration to a HTTP server.
+     */
+    public void testSaveToHTTPServerSuccess() throws Exception
+    {
+        MockHttpURLStreamHandler handler = new MockHttpURLStreamHandler(
+                HttpURLConnection.HTTP_OK, testSavePropertiesFile);
+        URL url = new URL(null, "http://jakarta.apache.org", handler);
+        conf.save(url);
+        MockHttpURLConnection con = handler.getMockConnection();
+        assertTrue("Wrong output flag", con.getDoOutput());
+        assertEquals("Wrong method", "PUT", con.getRequestMethod());
+
+        PropertiesConfiguration checkConfig = new PropertiesConfiguration(
+                testSavePropertiesFile);
+        ConfigurationAssert.assertEquals(conf, checkConfig);
+    }
+
+    /**
+     * Tests saving a file-based configuration to a HTTP server when the server
+     * reports a failure. This should cause an exception.
+     */
+    public void testSaveToHTTPServerFail() throws Exception
+    {
+        MockHttpURLStreamHandler handler = new MockHttpURLStreamHandler(
+                HttpURLConnection.HTTP_BAD_REQUEST, testSavePropertiesFile);
+        URL url = new URL(null, "http://jakarta.apache.org", handler);
+        try
+        {
+            conf.save(url);
+            fail("Response code was not checked!");
+        }
+        catch (ConfigurationException cex)
+        {
+            assertTrue("Wrong root cause: " + cex,
+                    cex.getCause() instanceof IOException);
+        }
+    }
+
+    /**
+     * Tests initializing a properties configuration from a non existing file.
+     * There was a bug, which caused properties getting lost when later save()
+     * is called.
+     */
+    public void testInitFromNonExistingFile() throws ConfigurationException
+    {
+        final String testProperty = "test.successfull";
+        conf = new PropertiesConfiguration(testSavePropertiesFile);
+        conf.addProperty(testProperty, Boolean.TRUE);
+        conf.save();
+        PropertiesConfiguration checkConfig = new PropertiesConfiguration(
+                testSavePropertiesFile);
+        assertTrue("Test property not found", checkConfig
+                .getBoolean(testProperty));
+    }
+
+    /**
+     * Tests copying another configuration into the test configuration. This
+     * test ensures that the layout object is informed about the newly added
+     * properties.
+     */
+    public void testCopyAndSave() throws ConfigurationException
+    {
+        Configuration copyConf = setUpCopyConfig();
+        conf.copy(copyConf);
+        checkCopiedConfig(copyConf);
+    }
+
+    /**
+     * Tests appending a configuration to the test configuration. Again it has
+     * to be ensured that the layout object is correctly updated.
+     */
+    public void testAppendAndSave() throws ConfigurationException
+    {
+        Configuration copyConf = setUpCopyConfig();
+        conf.append(copyConf);
+        checkCopiedConfig(copyConf);
+    }
+
+    /**
+     * Creates a configuration that can be used for testing copy operations.
+     *
+     * @return the configuration to be copied
+     */
+    private Configuration setUpCopyConfig()
+    {
+        final int count = 25;
+        Configuration result = new BaseConfiguration();
+        for (int i = 1; i <= count; i++)
+        {
+            result.addProperty("copyKey" + i, "copyValue" + i);
+        }
+        return result;
+    }
+
+    /**
+     * Tests whether the data of a configuration that was copied into the test
+     * configuration is correctly saved.
+     *
+     * @param copyConf the copied configuration
+     * @throws ConfigurationException if an error occurs
+     */
+    private void checkCopiedConfig(Configuration copyConf)
+            throws ConfigurationException
+    {
+        conf.save(testSavePropertiesFile);
+        PropertiesConfiguration checkConf = new PropertiesConfiguration(
+                testSavePropertiesFile);
+        for (Iterator it = copyConf.getKeys(); it.hasNext();)
+        {
+            String key = (String) it.next();
+            assertEquals("Wrong value for property " + key, checkConf
+                    .getProperty(key), copyConf.getProperty(key));
+        }
     }
 
     /**
@@ -715,4 +835,79 @@ public class TestPropertiesConfiguration extends TestCase
             loadCalls++;
         }
     }
+
+    /**
+     * A mock implementation of a HttpURLConnection used for testing saving to
+     * a HTTP server.
+     */
+    static class MockHttpURLConnection extends HttpURLConnection
+    {
+        /** The response code to return.*/
+        private int responseCode;
+
+        /** The output file. The output stream will point to this file.*/
+        private File outputFile;
+
+        protected MockHttpURLConnection(URL u, int respCode, File outFile)
+        {
+            super(u);
+            responseCode = respCode;
+            outputFile = outFile;
+        }
+
+        public void disconnect()
+        {
+        }
+
+        public boolean usingProxy()
+        {
+            return false;
+        }
+
+        public void connect() throws IOException
+        {
+        }
+
+        public int getResponseCode() throws IOException
+        {
+            return responseCode;
+        }
+
+        public OutputStream getOutputStream() throws IOException
+        {
+            return new FileOutputStream(outputFile);
+        }
+    }
+
+    /**
+     * A mock stream handler for working with the mock HttpURLConnection.
+     */
+    static class MockHttpURLStreamHandler extends URLStreamHandler
+    {
+        /** Stores the response code.*/
+        private int responseCode;
+
+        /** Stores the output file.*/
+        private File outputFile;
+
+        /** Stores the connection.*/
+        private MockHttpURLConnection connection;
+
+        public MockHttpURLStreamHandler(int respCode, File outFile)
+        {
+            responseCode = respCode;
+            outputFile = outFile;
+        }
+
+        public MockHttpURLConnection getMockConnection()
+        {
+            return connection;
+        }
+
+        protected URLConnection openConnection(URL u) throws IOException
+        {
+            connection = new MockHttpURLConnection(u, responseCode, outputFile);
+            return connection;
+        }
+    }
 }
