@@ -26,14 +26,17 @@ import java.io.PrintWriter;
 import java.io.StringReader;
 import java.io.StringWriter;
 import java.net.URL;
+import java.util.ArrayList;
+import java.util.Collection;
 import java.util.Iterator;
 import java.util.List;
 
 import javax.xml.parsers.DocumentBuilder;
 import javax.xml.parsers.DocumentBuilderFactory;
 
+import junit.framework.TestCase;
+
 import org.apache.commons.configuration.reloading.FileAlwaysReloadingStrategy;
-import org.apache.commons.configuration.reloading.FileChangedReloadingStrategy;
 import org.apache.commons.configuration.reloading.InvariantReloadingStrategy;
 import org.apache.commons.configuration.tree.ConfigurationNode;
 import org.apache.commons.configuration.tree.xpath.XPathExpressionEngine;
@@ -41,12 +44,10 @@ import org.xml.sax.SAXException;
 import org.xml.sax.SAXParseException;
 import org.xml.sax.helpers.DefaultHandler;
 
-import junit.framework.TestCase;
-
 /**
  * test for loading and saving xml properties files
  *
- * @version $Id: TestXMLConfiguration.java 513498 2007-03-01 21:15:07Z oheger $
+ * @version $Id: TestXMLConfiguration.java 578579 2007-09-23 16:20:24Z oheger $
  */
 public class TestXMLConfiguration extends TestCase
 {
@@ -264,7 +265,7 @@ public class TestXMLConfiguration extends TestCase
         // set a new attribute
         conf.setProperty("foo[@bar]", "value");
         assertEquals("foo[@bar]", "value", conf.getProperty("foo[@bar]"));
-        
+
         conf.setProperty("name1","value1");
         assertEquals("value1",conf.getProperty("name1"));
     }
@@ -286,6 +287,40 @@ public class TestXMLConfiguration extends TestCase
         assertTrue("test.boolean[@value]", conf.getBoolean("test.boolean[@value]"));
     }
 
+    /**
+     * Tests setting an attribute on the root element.
+     */
+    public void testSetRootAttribute() throws ConfigurationException
+    {
+        conf.setProperty("[@test]", "true");
+        assertEquals("Root attribute not set", "true", conf
+                .getString("[@test]"));
+        conf.save(testSaveConf);
+        XMLConfiguration checkConf = new XMLConfiguration();
+        checkConf.setFile(testSaveConf);
+        checkSavedConfig(checkConf);
+        assertTrue("Attribute not found after save", checkConf
+                .containsKey("[@test]"));
+        checkConf.setProperty("[@test]", "newValue");
+        checkConf.save();
+        conf = checkConf;
+        checkConf = new XMLConfiguration();
+        checkConf.setFile(testSaveConf);
+        checkSavedConfig(checkConf);
+        assertEquals("Attribute not modified after save", "newValue", checkConf
+                .getString("[@test]"));
+    }
+
+    /**
+     * Tests whether the configuration's root node is initialized with a
+     * reference to the corresponding XML element.
+     */
+    public void testGetRootReference()
+    {
+        assertNotNull("Root node has no reference", conf.getRootNode()
+                .getReference());
+    }
+
     public void testAddList()
     {
         conf.addProperty("test.array", "value1");
@@ -353,11 +388,11 @@ public class TestXMLConfiguration extends TestCase
         assertTrue(conf.isEmpty());
         conf.addProperty("test", "yes");
         conf.save();
-        
+
         conf = new XMLConfiguration(testSaveConf);
         assertEquals("yes", conf.getString("test"));
     }
-    
+
     /**
      * Tests loading a configuration from a URL.
      */
@@ -368,7 +403,7 @@ public class TestXMLConfiguration extends TestCase
         assertEquals("value", conf.getProperty("element"));
         assertEquals(url, conf.getURL());
     }
-    
+
     /**
      * Tests loading from a stream.
      */
@@ -378,12 +413,12 @@ public class TestXMLConfiguration extends TestCase
         conf = new XMLConfiguration();
         conf.load(new ByteArrayInputStream(xml.getBytes()));
         assertEquals(1, conf.getInt("test"));
-        
+
         conf = new XMLConfiguration();
         conf.load(new ByteArrayInputStream(xml.getBytes()), "UTF8");
         assertEquals(1, conf.getInt("test"));
     }
-    
+
     /**
      * Tests loading a non well formed XML from a string.
      */
@@ -440,7 +475,7 @@ public class TestXMLConfiguration extends TestCase
         {
            conf.addProperty("test.attribute[@array]", "value" + i);
         }
-        
+
         // add comma delimited lists with escaped delimiters
         conf.addProperty("split.list5", "a\\,b\\,c");
         conf.setProperty("element3", "value\\,value1\\,value2");
@@ -454,7 +489,7 @@ public class TestXMLConfiguration extends TestCase
         checkConfig.setFileName(testSaveConf.getAbsolutePath());
         checkSavedConfig(checkConfig);
     }
-    
+
     /**
      * Tests saving to a URL.
      */
@@ -465,7 +500,7 @@ public class TestXMLConfiguration extends TestCase
         checkConfig.setFile(testSaveConf);
         checkSavedConfig(checkConfig);
     }
-    
+
     /**
      * Tests saving to a stream.
      */
@@ -486,11 +521,11 @@ public class TestXMLConfiguration extends TestCase
                 out.close();
             }
         }
-        
+
         XMLConfiguration checkConfig = new XMLConfiguration();
         checkConfig.setFile(testSaveConf);
         checkSavedConfig(checkConfig);
-        
+
         try
         {
             out = new FileOutputStream(testSaveConf);
@@ -503,14 +538,14 @@ public class TestXMLConfiguration extends TestCase
                 out.close();
             }
         }
-        
+
         checkConfig.clear();
         checkSavedConfig(checkConfig);
     }
 
     public void testAutoSave() throws Exception
     {
-        conf.setFile(new File("target/testsave.xml"));
+        conf.setFile(testSaveConf);
         assertFalse(conf.isAutoSave());
         conf.setAutoSave(true);
         assertTrue(conf.isAutoSave());
@@ -519,13 +554,13 @@ public class TestXMLConfiguration extends TestCase
         // reload the configuration
         XMLConfiguration conf2 = new XMLConfiguration(conf.getFile());
         assertEquals("'autosave' property", "ok", conf2.getString("autosave"));
-        
+
         conf.clearTree("clear");
         conf2 = new XMLConfiguration(conf.getFile());
         Configuration sub = conf2.subset("clear");
         assertTrue(sub.isEmpty());
     }
-    
+
     /**
      * Tests if a second file can be appended to a first.
      */
@@ -537,14 +572,14 @@ public class TestXMLConfiguration extends TestCase
         conf.load(testProperties2);
         assertEquals("value", conf.getString("element"));
         assertEquals("tasks", conf.getString("table.name"));
-        
+
         conf.save(testSaveConf);
         conf = new XMLConfiguration(testSaveConf);
         assertEquals("value", conf.getString("element"));
         assertEquals("tasks", conf.getString("table.name"));
         assertEquals("application", conf.getString("table[@tableType]"));
     }
-    
+
     /**
      * Tests saving attributes (related to issue 34442).
      */
@@ -750,12 +785,12 @@ public class TestXMLConfiguration extends TestCase
         File nonValidFile = new File("conf/testValidateInvalid.xml");
         conf = new XMLConfiguration();
         assertFalse(conf.isValidating());
-        
+
         // Load a non valid XML document. Should work for isValidating() == false
         conf.load(nonValidFile);
         assertEquals("customers", conf.getString("table.name"));
         assertFalse(conf.containsKey("table.fields.field(1).type"));
-        
+
         // Now set the validating flag to true
         conf.setValidating(true);
         try
@@ -768,7 +803,7 @@ public class TestXMLConfiguration extends TestCase
             //ok
         }
     }
-    
+
     /**
      * Tests handling of empty elements.
      */
@@ -779,12 +814,12 @@ public class TestXMLConfiguration extends TestCase
         conf.addProperty("empty2", "");
         conf.setProperty("empty", "no more empty");
         conf.save(testSaveConf);
-        
+
         conf = new XMLConfiguration(testSaveConf);
         assertEquals("no more empty", conf.getString("empty"));
         assertEquals("", conf.getProperty("empty2"));
     }
-    
+
     /**
      * Tests whether the encoding is correctly detected by the XML parser. This
      * is done by loading an XML file with the encoding "UTF-16". If this
@@ -799,7 +834,7 @@ public class TestXMLConfiguration extends TestCase
         conf.load(file);
         assertEquals("test3_yoge", conf.getString("yoge"));
     }
-    
+
     /**
      * Tests whether the encoding is written to the generated XML file.
      */
@@ -814,7 +849,7 @@ public class TestXMLConfiguration extends TestCase
         assertTrue("Encoding was not written to file", out.toString().indexOf(
                 "encoding=\"" + ENCODING + "\"") >= 0);
     }
-    
+
     /**
      * Tests whether a default encoding is used if no specific encoding is set.
      * According to the XSLT specification (http://www.w3.org/TR/xslt#output)
@@ -939,7 +974,7 @@ public class TestXMLConfiguration extends TestCase
         conf.clear();
         assertNull(conf.getString("test[1]/entity/@name"));
     }
-    
+
     /**
      * Tests the copy constructor.
      */
@@ -962,6 +997,287 @@ public class TestXMLConfiguration extends TestCase
         checkSavedConfig(copy);
     }
 
+    /**
+     * Tests list nodes with multiple values and attributes.
+     */
+    public void testListWithAttributes()
+    {
+        assertEquals("Wrong number of <a> elements", 6, conf.getList(
+                "attrList.a").size());
+        assertEquals("Wrong value of first element", "ABC", conf
+                .getString("attrList.a(0)"));
+        assertEquals("Wrong value of first name attribute", "x", conf
+                .getString("attrList.a(0)[@name]"));
+        assertEquals("Wrong number of name attributes", 5, conf.getList(
+                "attrList.a[@name]").size());
+    }
+
+    /**
+     * Tests a list node with attributes that has multiple values separated by
+     * the list delimiter. In this scenario the attribute should be added to the
+     * node with the first value.
+     */
+    public void testListWithAttributesMultiValue()
+    {
+        assertEquals("Wrong value of 2nd element", "1", conf
+                .getString("attrList.a(1)"));
+        assertEquals("Wrong value of 2nd name attribute", "y", conf
+                .getString("attrList.a(1)[@name]"));
+        for (int i = 2; i <= 3; i++)
+        {
+            assertEquals("Wrong value of element " + (i + 1), i, conf
+                    .getInt("attrList.a(" + i + ")"));
+            assertFalse("element " + (i + 1) + " has attribute", conf
+                    .containsKey("attrList.a(2)[@name]"));
+        }
+    }
+
+    /**
+     * Tests a list node with a multi-value attribute and multiple values. All
+     * attribute values should be assigned to the node with the first value.
+     */
+    public void testListWithMultiAttributesMultiValue()
+    {
+        for (int i = 1; i <= 2; i++)
+        {
+            assertEquals("Wrong value of multi-valued node", "value" + i, conf
+                    .getString("attrList.a(" + (i + 3) + ")"));
+        }
+        List attrs = conf.getList("attrList.a(4)[@name]");
+        final String attrVal = "uvw";
+        assertEquals("Wrong number of name attributes", attrVal.length(), attrs
+                .size());
+        for (int i = 0; i < attrVal.length(); i++)
+        {
+            assertEquals("Wrong value for attribute " + i, String
+                    .valueOf(attrVal.charAt(i)), attrs.get(i));
+        }
+        assertEquals("Wrong value of test attribute", "yes", conf
+                .getString("attrList.a(4)[@test]"));
+        assertFalse("Name attribute for 2nd value", conf
+                .containsKey("attrList.a(5)[@name]"));
+        assertFalse("Test attribute for 2nd value", conf
+                .containsKey("attrList.a(5)[@test]"));
+    }
+
+    /**
+     * Tests whether the auto save mechanism is triggered by changes at a
+     * subnode configuration.
+     */
+    public void testAutoSaveWithSubnodeConfig() throws ConfigurationException
+    {
+        final String newValue = "I am autosaved";
+        conf.setFile(testSaveConf);
+        conf.setAutoSave(true);
+        Configuration sub = conf.configurationAt("element2.subelement");
+        sub.setProperty("subsubelement", newValue);
+        assertEquals("Change not visible to parent", newValue, conf
+                .getString("element2.subelement.subsubelement"));
+        XMLConfiguration conf2 = new XMLConfiguration(testSaveConf);
+        assertEquals("Change was not saved", newValue, conf2
+                .getString("element2.subelement.subsubelement"));
+    }
+
+    /**
+     * Tests whether a subnode configuration created from another subnode
+     * configuration of a XMLConfiguration can trigger the auto save mechanism.
+     */
+    public void testAutoSaveWithSubSubnodeConfig() throws ConfigurationException
+    {
+        final String newValue = "I am autosaved";
+        conf.setFile(testSaveConf);
+        conf.setAutoSave(true);
+        SubnodeConfiguration sub1 = conf.configurationAt("element2");
+        SubnodeConfiguration sub2 = sub1.configurationAt("subelement");
+        sub2.setProperty("subsubelement", newValue);
+        assertEquals("Change not visible to parent", newValue, conf
+                .getString("element2.subelement.subsubelement"));
+        XMLConfiguration conf2 = new XMLConfiguration(testSaveConf);
+        assertEquals("Change was not saved", newValue, conf2
+                .getString("element2.subelement.subsubelement"));
+    }
+
+    /**
+     * Tests saving and loading a configuration when delimiter parsing is
+     * disabled.
+     */
+    public void testSaveDelimiterParsingDisabled()
+            throws ConfigurationException
+    {
+        checkSaveDelimiterParsingDisabled("list.delimiter.test");
+    }
+
+    /**
+     * Tests saving and loading a configuration when delimiter parsing is
+     * disabled and attributes are involved.
+     */
+    public void testSaveDelimiterParsingDisabledAttrs()
+            throws ConfigurationException
+    {
+        checkSaveDelimiterParsingDisabled("list.delimiter.test[@attr]");
+    }
+
+    /**
+     * Helper method for testing saving and loading a configuration when
+     * delimiter parsing is disabled.
+     *
+     * @param key the key to be checked
+     * @throws ConfigurationException if an error occurs
+     */
+    private void checkSaveDelimiterParsingDisabled(String key)
+            throws ConfigurationException
+    {
+        conf.clear();
+        conf.setDelimiterParsingDisabled(true);
+        conf.load();
+        conf.setProperty(key, "C:\\Temp\\,C:\\Data\\");
+        conf.addProperty(key, "a,b,c");
+        conf.save(testSaveConf);
+        XMLConfiguration checkConf = new XMLConfiguration();
+        checkConf.setDelimiterParsingDisabled(true);
+        checkConf.setFile(testSaveConf);
+        checkSavedConfig(checkConf);
+    }
+
+    /**
+     * Tests multiple attribute values in delimiter parsing disabled mode.
+     */
+    public void testDelimiterParsingDisabledMultiAttrValues() throws ConfigurationException
+    {
+        conf.clear();
+        conf.setDelimiterParsingDisabled(true);
+        conf.load();
+        List expr = conf.getList("expressions[@value]");
+        assertEquals("Wrong list size", 2, expr.size());
+        assertEquals("Wrong element 1", "a || (b && c)", expr.get(0));
+        assertEquals("Wrong element 2", "!d", expr.get(1));
+    }
+
+    /**
+     * Tests using multiple attribute values, which are partly escaped when
+     * delimiter parsing is not disabled.
+     */
+    public void testMultipleAttrValuesEscaped() throws ConfigurationException
+    {
+        conf.addProperty("test.dir[@name]", "C:\\Temp\\");
+        conf.addProperty("test.dir[@name]", "C:\\Data\\");
+        conf.save(testSaveConf);
+        XMLConfiguration checkConf = new XMLConfiguration();
+        checkConf.setFile(testSaveConf);
+        checkSavedConfig(checkConf);
+    }
+
+    /**
+     * Tests a combination of auto save = true and an associated reloading
+     * strategy.
+     */
+    public void testAutoSaveWithReloadingStrategy() throws ConfigurationException
+    {
+        conf.setFile(testSaveConf);
+        conf.save();
+        conf.setReloadingStrategy(new FileAlwaysReloadingStrategy());
+        conf.setAutoSave(true);
+        assertEquals("Value not found", "value", conf.getProperty("element"));
+    }
+
+    /**
+     * Tests adding nodes from another configuration.
+     */
+    public void testAddNodesCopy() throws ConfigurationException
+    {
+        XMLConfiguration c2 = new XMLConfiguration(testProperties2);
+        conf.addNodes("copiedProperties", c2.getRootNode().getChildren());
+        conf.save(testSaveConf);
+        XMLConfiguration checkConf = new XMLConfiguration();
+        checkConf.setFile(testSaveConf);
+        checkSavedConfig(checkConf);
+    }
+
+    /**
+     * Tests whether the addNodes() method triggers an auto save.
+     */
+    public void testAutoSaveAddNodes() throws ConfigurationException
+    {
+        conf.setFile(testSaveConf);
+        conf.setAutoSave(true);
+        HierarchicalConfiguration.Node node = new HierarchicalConfiguration.Node(
+                "addNodesTest", Boolean.TRUE);
+        Collection nodes = new ArrayList(1);
+        nodes.add(node);
+        conf.addNodes("test.autosave", nodes);
+        XMLConfiguration c2 = new XMLConfiguration(testSaveConf);
+        assertTrue("Added nodes are not saved", c2
+                .getBoolean("test.autosave.addNodesTest"));
+    }
+
+    /**
+     * Tests saving a configuration after a node was added. Test for
+     * CONFIGURATION-294.
+     */
+    public void testAddNodesAndSave() throws ConfigurationException
+    {
+        ConfigurationNode node = new HierarchicalConfiguration.Node("test");
+        ConfigurationNode child = new HierarchicalConfiguration.Node("child");
+        node.addChild(child);
+        ConfigurationNode attr = new HierarchicalConfiguration.Node("attr");
+        node.addAttribute(attr);
+        ConfigurationNode node2 = conf.createNode("test2");
+        Collection nodes = new ArrayList(2);
+        nodes.add(node);
+        nodes.add(node2);
+        conf.addNodes("add.nodes", nodes);
+        conf.setFile(testSaveConf);
+        conf.save();
+        conf.setProperty("add.nodes.test", "true");
+        conf.setProperty("add.nodes.test.child", "yes");
+        conf.setProperty("add.nodes.test[@attr]", "existing");
+        conf.setProperty("add.nodes.test2", "anotherValue");
+        conf.save();
+        XMLConfiguration c2 = new XMLConfiguration(testSaveConf);
+        assertEquals("Value was not saved", "true", c2
+                .getString("add.nodes.test"));
+        assertEquals("Child value was not saved", "yes", c2
+                .getString("add.nodes.test.child"));
+        assertEquals("Attr value was not saved", "existing", c2
+                .getString("add.nodes.test[@attr]"));
+        assertEquals("Node2 not saved", "anotherValue", c2
+                .getString("add.nodes.test2"));
+    }
+
+    /**
+     * Tests registering the publicId of a DTD.
+     */
+    public void testRegisterEntityId() throws ConfigurationException,
+            IOException
+    {
+        File dtdFile = new File("conf/properties.dtd");
+        final String publicId = "http://commons.apache.org/test/properties.dtd";
+        conf = new XMLConfiguration("testDtd.xml");
+        conf.setPublicID(publicId);
+        conf.save(testSaveConf);
+        XMLConfiguration checkConfig = new XMLConfiguration();
+        checkConfig.setFile(testSaveConf);
+        checkConfig.registerEntityId(publicId, dtdFile.toURL());
+        checkConfig.setValidating(true);
+        checkSavedConfig(checkConfig);
+    }
+
+    /**
+     * Tries to register a null public ID. This should cause an exception.
+     */
+    public void testRegisterEntityIdNull() throws IOException
+    {
+        try
+        {
+            conf.registerEntityId(null, new URL("http://commons.apache.org"));
+            fail("Could register null public ID!");
+        }
+        catch (IllegalArgumentException iex)
+        {
+            // ok
+        }
+    }
+
     /**
      * Prepares a configuration object for testing a reload operation.
      *
@@ -973,14 +1289,7 @@ public class TestXMLConfiguration extends TestCase
         removeTestFile();
         conf.save(testSaveConf);
         XMLConfiguration c = new XMLConfiguration(testSaveConf);
-        c.setReloadingStrategy(new FileChangedReloadingStrategy()
-        {
-            // Report always a change
-            protected boolean hasChanged()
-            {
-                return true;
-            }
-        });
+        c.setReloadingStrategy(new FileAlwaysReloadingStrategy());
         conf.setProperty("test(0).entity", "newValue");
         conf.save(testSaveConf);
         return c;
@@ -1006,12 +1315,6 @@ public class TestXMLConfiguration extends TestCase
     private void checkSavedConfig(FileConfiguration checkConfig) throws ConfigurationException
     {
         checkConfig.load();
-
-        for (Iterator i = conf.getKeys(); i.hasNext();)
-        {
-            String key = (String) i.next();
-            assertTrue("The saved configuration doesn't contain the key '" + key + "'", checkConfig.containsKey(key));
-            assertEquals("Value of the '" + key + "' property", conf.getProperty(key), checkConfig.getProperty(key));
-        }
+        ConfigurationAssert.assertEquals(conf, checkConfig);
     }
 }
